﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using WeifenLuo.WinFormsUI.Docking;
using LibBinaryRelation.Controls;

namespace BinaryRelation
{
    public partial class Content : DockContent
    {
        public Content()
        {
            InitializeComponent();

            viewer = new PNGViewer1();
            viewer.BorderStyle = BorderStyle.Fixed3D;
            viewer.Dock = DockStyle.Fill;
            List<string> b = new List<string>();
            for (int i = 0; i < 17; i++) b.Add(Application.StartupPath + "\\Images\\Page" + i + ".png");
            this.viewer.SetPNGs(b);

            preambl = new Preamble();
            preambl.BorderStyle = BorderStyle.Fixed3D;
            preambl.Dock = DockStyle.Fill;
            name_item = "Úvod";

            l = new List<UserControl>() 
            {
                new RepreRelationGraphX(),
                new CartesianProduct(),
                new BinaryRelationOnSet(),
                new RepreRelationMatrix(),
                new RepreRelationSetGraphMatrix(),
                new IntersectionRelation(),
                new InverseRelation(),
                new UnionRelation(),
                new ReflexiveRelation(),
                new SymmetryRelation(),
                new TransitiveRelation(),
                new Closures(),
                new EquivalenceRelation(),
                new Coverage(),
                new OrderRelation1(),
                new OrderRelation2(),
                new OrderRelation3()
            };
            l.ForEach(item => { item.BorderStyle = BorderStyle.Fixed3D; item.Dock = DockStyle.Fill; });
        }

        PNGViewer1 viewer;
        Preamble preambl;
        string name_item;
        List<UserControl> l;

        public event DelegateChanged ChangedItem;
        public event DelegateChanged ChangedDock;

        public PNGViewer1 Viewer1
        {
            get { return this.viewer; }
        }

        public string NameItem
        {
            get { return name_item; }
        }

        public void SetTrieViewBackColor(Color color)
        {
            this.treeView1.BackColor = color;
        }

        private void treeView1AfterSelect(object sender, TreeViewEventArgs e)
        {
            try
            {
                if (e.Node.Tag != null && ((string)e.Node.Tag).Length != 0)
                {
                    UserControl c;
                    int item = int.Parse((string)e.Node.Tag);
                    name_item = e.Node.FullPath;
                    switch (item)
                    {
                        case 0: ChangedItem(preambl); break;
                        case 1: viewer.SetCurrentItem(4); ChangedItem(viewer); break;
                        case 2: ChangedItem(l[1]);break;
                        case 3: ChangedItem(l[2]);break;
                        case 4: viewer.SetCurrentItem(5); viewer.SetAutoScrollPositionDown(); ChangedItem(viewer); break;
                        case 5: ChangedItem(l[3]); break;
                        case 6: ChangedItem(l[0]); break;
                        case 7: ChangedItem(l[4]); break;
                        case 8: viewer.SetCurrentItem(5); ChangedItem(viewer); break;
                        case 9: ChangedItem(l[5]); break;
                        case 10: ChangedItem(l[7]); break;
                        case 11: ChangedItem(l[6]); break;
                        case 16: viewer.SetCurrentItem(8); viewer.SetAutoScrollPositionDown(); ChangedItem(viewer); break;
                        case 17: ChangedItem(l[8]); break;
                        case 18: ChangedItem(l[9]); break;
                        case 19: ChangedItem(l[10]); break;
                        case 20: ChangedItem(l[11]); break;
                        case 21: viewer.SetCurrentItem(14); ChangedItem(viewer); break;
                        case 22: ChangedItem(l[12]); break;
                        case 23: viewer.SetCurrentItem(12); ChangedItem(viewer); break;
                        case 24: ChangedItem(l[13]); break;
                        case 25: ChangedItem(l[14]); break;
                        case 26: ChangedItem(l[15]); break;
                        case 27: ChangedItem(l[16]); break;
                        default: ChangedItem(new UserControl()); break;
                    }
                }
            }
            catch (Exception)
            {
                MessageBox.Show("Soubor nebyl nalezen", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void toolStripMenuItem1Click(object sender, EventArgs e)
        {
            this.ShowHint = DockState.Float;
            ChangedDock(this);
        }
    }
}
