﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace LibBinaryRelation
{
    public class Computing
    {
        public static Relation UnionRelations(string name_new_relation, params Relation[] relations)
        {
            Relation u = new Relation(name_new_relation);
            for (int i = 0; i < relations[0].CountItemsInRelation(); i++)
            {
                u.AddItemToRelation(relations[0].GetItemOfItemOfRelation(i, 0), relations[0].GetItemOfItemOfRelation(i, 1), relations[0].GetColorItemOfRelation(i));
            }

            for (int j = 1; j < relations.Count(); j++)
            {
                for (int i = 0; i < relations[j].CountItemsInRelation(); i++)
                {
                    int first_item = relations[j].GetItemOfItemOfRelation(i, 0);
                    int second_item = relations[j].GetItemOfItemOfRelation(i, 1);
                    if (u.GetNumberOfItemOfRelation(first_item, second_item) == -1) u.AddItemToRelation(first_item, second_item, relations[j].GetColorItemOfRelation(i));
                }
            }

            return u;
        }

        public static Relation CompositionRelations(Relation r1, Relation r2, string name_new_relation)
        {
            Relation c = new Relation(name_new_relation);

            for (int i = 0; i < r1.CountItemsInRelation(); i++)
            {
                int second_item = r1.GetItemOfItemOfRelation(i,1);

                for (int j = 0; j < r2.CountItemsInRelation(); j++)
                {
                    if (second_item == r2.GetItemOfItemOfRelation(j, 0)) c.AddItemToRelation(r1.GetItemOfItemOfRelation(i, 0), r2.GetItemOfItemOfRelation(j, 1), r1.GetColorItemOfRelation(i));
                }
            }

            return c;
        }

        public static Relation InverseRelation(Relation r, string name_new_relation)
        {
            Relation i_r = new Relation(name_new_relation);
            for (int i = 0; i < r.CountItemsInRelation(); i++) i_r.AddItemToRelation(r.GetItemOfItemOfRelation(i, 1), r.GetItemOfItemOfRelation(i, 0), r.GetColorItemOfRelation(i));

            return i_r;
        }

        public static Relation ReflexiveCRelation(Relation r, Set set, string name_new_relation)
        {
            Relation identity = new Relation("identity");
            for (int i = 0; i < set.CountItemsInSet(); i++) identity.AddItemToRelation(i, i,Brushes.Black);
            return UnionRelations(name_new_relation, r, identity);
        }

        public static Relation SymmetryCRelation(Relation r, string name_new_relation)
        {
            return UnionRelations(name_new_relation, r, InverseRelation(r, ""));
        }

        public static Relation TransitiveCRelation(Relation r, Set set, string name_new_relation)
        {
            Relation[] re = new Relation[set.CountItemsInSet()];
            re[0] = r;
            for (int i = 1; i < set.CountItemsInSet(); i++) re[i] = CompositionRelations(re[i - 1], r, "");

            return UnionRelations(name_new_relation, re);
        }

        public static bool IsReflexive(Relation r, Set set)
        {
            for (int i = 0; i < set.CountItemsInSet(); i++)
            {
                if (r.GetNumberOfItemOfRelation(i, i) == -1) return false;
            }

            return true;
        }

        public static bool IsAntiSymmetry(Relation r)
        {
            int first_item;
            int second_item;
            for (int i = 0; i < r.CountItemsInRelation(); i++)
            {
                first_item = r.GetItemOfItemOfRelation(i,0);
                second_item = r.GetItemOfItemOfRelation(i,1);
                if (first_item != second_item && r.GetNumberOfItemOfRelation(second_item, first_item) != -1) return false;
            }
            return true;
        }

        public static bool IsSymmetry(Relation r)
        {
            for (int i = 0; i < r.CountItemsInRelation(); i++)
            {
                if (r.GetNumberOfItemOfRelation(r.GetItemOfItemOfRelation(i, 1), r.GetItemOfItemOfRelation(i, 0)) == -1) return false;
            }

            return true;
        }

        public static bool IsTransitive(Relation r)
        {
            for (int i = 0; i < r.CountItemsInRelation(); i++)
            {
                int second_item = r.GetItemOfItemOfRelation(i, 1);

                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 0) == second_item)
                    {
                        if (r.GetNumberOfItemOfRelation(r.GetItemOfItemOfRelation(i, 0), r.GetItemOfItemOfRelation(j, 1)) == -1) return false;
                    }
                }
            }

            return true;
        }

        public static Relation Identity(Set set, string name_relation)
        {
            Relation r = new Relation(name_relation);
            for (int i = 0; i < set.CountItemsInSet(); i++) r.AddItemToRelation(i, i, Brushes.Black);
            return r;
        }

        public static Relation CoverageRelation(Relation r, string name_new_relation)
        {
            Relation c_r = new Relation(name_new_relation);
            List<int> second_items = new List<int>();
            List<int> first_items = new List<int>();

            for (int i = 0; i < r.CountItemsInRelation(); i++)
            {
                int first_item = r.GetItemOfItemOfRelation(i,0);
                int second_item = r.GetItemOfItemOfRelation(i,1);

                if (first_item != second_item)
                {
                    for (int j = 0; j < r.CountItemsInRelation(); j++)
                    {
                        if(r.GetItemOfItemOfRelation(j,0) == first_item) second_items.Add(r.GetItemOfItemOfRelation(j,1));
                        if (r.GetItemOfItemOfRelation(j, 1) == second_item) first_items.Add(r.GetItemOfItemOfRelation(j, 0));
                    }

                    List<int> intersection = first_items.Intersect(second_items).ToList();
                    if (intersection.Count == 0 || intersection.All(item => item == first_item || item == second_item)) c_r.AddItemToRelation(first_item, second_item, Brushes.Black);

                    second_items.Clear(); first_items.Clear();
                }
            }

            return c_r;
        }

        //Order relation

        public static List<string> MinimalItem(Relation r, Set set)
        {
            List<string> minimal_items = new List<string>();
            bool is_not_minimal_item = false;

            for (int i = 0; i < set.CountItemsInSet(); i++)
            {
                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 1) == i && r.GetItemOfItemOfRelation(j, 0) != i) 
                    { 
                        is_not_minimal_item = true; break; 
                    }
                }

                if (!is_not_minimal_item) minimal_items.Add(set.GetNameItem(i));

                is_not_minimal_item = false;
            }

            return minimal_items;
        }

        public static List<int> MinimalItem1(Relation r, Set set)
        {
            List<int> minimal_items = new List<int>();
            bool is_not_minimal_item = false;

            for (int i = 0; i < set.CountItemsInSet(); i++)
            {
                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 1) == i && r.GetItemOfItemOfRelation(j, 0) != i)
                    {
                        is_not_minimal_item = true; break;
                    }
                }

                if (!is_not_minimal_item) minimal_items.Add(i);

                is_not_minimal_item = false;
            }

            return minimal_items;
        }

        public static List<string> MaximalItem(Relation r, Set set)
        {
            List<string> minimal_items = new List<string>();
            bool is_not_minimal_item = false;

            for (int i = 0; i < set.CountItemsInSet(); i++)
            {
                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 0) == i && r.GetItemOfItemOfRelation(j, 1) != i)
                    {
                        is_not_minimal_item = true; break;
                    }
                }

                if (!is_not_minimal_item) minimal_items.Add(set.GetNameItem(i));

                is_not_minimal_item = false;
            }

            return minimal_items;
        }

        public static List<int> MaximalItem1(Relation r, Set set)
        {
            List<int> minimal_items = new List<int>();
            bool is_not_minimal_item = false;

            for (int i = 0; i < set.CountItemsInSet(); i++)
            {
                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 0) == i && r.GetItemOfItemOfRelation(j, 1) != i)
                    {
                        is_not_minimal_item = true; break;
                    }
                }

                if (!is_not_minimal_item) minimal_items.Add(i);

                is_not_minimal_item = false;
            }

            return minimal_items;
        }

        public static string SmallestItem(Relation r, Set set)
        {
            int count_items = set.CountItemsInSet();
            int counter = count_items;

            for (int i = 0; i < count_items; i++)
            {
                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 0) == i) counter--;
                }

                if (counter == 0) return set.GetNameItem(i);
                else counter = count_items;
            }

            return null;
        }

        public static string BiggestItem(Relation r, Set set)
        {
            int count_items = set.CountItemsInSet();
            int counter = count_items;

            for (int i = 0; i < count_items; i++)
            {
                for (int j = 0; j < r.CountItemsInRelation(); j++)
                {
                    if (r.GetItemOfItemOfRelation(j, 1) == i) counter--;
                }

                if (counter == 0) return set.GetNameItem(i);
                else counter = count_items;
            }

            return null;
        }

        public static List<int> UpperCone(int count_items_set, Relation relation, List<int> subset_of_set)
        {
            List<int> upper_cone = new List<int>();
       
            int count_items_subset = subset_of_set.Count;
            if (count_items_subset != 0)
            {
                for (int i = 0; i < count_items_set; i++)
                    if (subset_of_set.Intersect(relation.GetItemsInRelationWith(i, 1)).Count() >= count_items_subset) upper_cone.Add(i);
            }

            return upper_cone; 
        }

        public static List<int> LowerCone(int count_items_set, Relation relation, List<int> subset_of_set)
        {
            List<int> upper_cone = new List<int>();
            int count_items_subset = subset_of_set.Count;
            if (count_items_subset != 0)
            {
                for (int i = 0; i < count_items_set; i++)
                    if (subset_of_set.Intersect(relation.GetItemsInRelationWith(i, 0)).Count() >= count_items_subset) upper_cone.Add(i);
            }

            return upper_cone;
        }

        /*public static Relation RandomOrderRelation(Set set, string name_relation)
        {
            Random random = new Random();
            Relation r = new Relation("");
            Relation r1;
            int max = set.CountItemsInSet();
            int count_items = max % 2 == 0 ? max * max / 2 + max / 2 : max * (max + 1) / 2;
            count_items = random.Next(count_items + 1);
            for (int i = 0; i < set.CountItemsInSet(); i++) r.AddItemToRelation(i, i, Brushes.Black);
            for (int j = 0; j < count_items; j++)
            {
                int first_random_item = random.Next(max);
                int second_random_item = random.Next(max);
                if (r.GetNumberOfItemOfRelation(second_random_item, first_random_item) == -1)
                {
                    r.AddItemToRelation(first_random_item, second_random_item, Brushes.Black);
                    r1 = Computing.TransitiveCRelation(r, set, "");
                    if (Computing.IsAntiSymmetry(r1)) r = r1;
                    else r.RemoveItemFromRelation(first_random_item, second_random_item);
                }
            }
            r.NameRelation = name_relation;
            return r;
        }*/

        public static Relation CreateRandomOrderRelation(Set set, string name_relation)
        {
            Random random = new Random();
            Relation r = new Relation(name_relation);
            r = Computing.ReflexiveCRelation(r, set, name_relation);
            Relation help_r = Computing.ReflexiveCRelation(r, set, name_relation);
            int count_items_in_set = set.CountItemsInSet();
            int max_items = ((count_items_in_set * count_items_in_set) - count_items_in_set) / 2;
            int random_max_items = random.Next(max_items + 1);
            int item1, item2;
            for (int i = 0; i < random_max_items; i++)
            {
                item1 = random.Next(count_items_in_set);
                item2 = random.Next(count_items_in_set);
                if (r.GetNumberOfItemOfRelation(item1, item2) == -1)
                {
                    if (r.GetNumberOfItemOfRelation(item2, item1) == -1)
                    {
                        help_r.AddItemToRelation(item1, item2, Brushes.Black);
                        List<int> items_item2 = help_r.GetItemsInRelationWith(item2, 0);
                        List<int> items_item1 = help_r.GetItemsInRelationWith(item1, 1);
                        items_item2.ForEach(item => help_r.AddItemToRelation(item1, item, Brushes.Black));
                        items_item1.ForEach(item => help_r.AddItemToRelation(item, item2, Brushes.Black));
                        if (Computing.IsAntiSymmetry(help_r) && Computing.IsTransitive(help_r)) r = help_r.Clone();
                        else help_r = r.Clone();
                    }
                }
            }
            return r;
        }

        public static Relation CreateRandomEquivalenceRelation(Set set, string name_relation)
        {
            Random random = new Random();
            Relation r = new Relation(name_relation);
            r = Computing.ReflexiveCRelation(r, set, name_relation);
            Relation help_r = Computing.ReflexiveCRelation(r, set, name_relation);
            int count_items_in_set = set.CountItemsInSet();
            int max_items = count_items_in_set * count_items_in_set;
            int random_max_items = random.Next(max_items + 1);
            int item1, item2;
            List<int> items_item2 = new List<int>();
            List<int> items_item1 = new List<int>();
            for (int i = 0; i < random_max_items; i++)
            {
                items_item1.Clear(); items_item2.Clear();
                item1 = random.Next(count_items_in_set);
                item2 = random.Next(count_items_in_set);

                help_r.AddItemToRelation(item1, item2, Brushes.Black);
                help_r.AddItemToRelation(item2, item1, Brushes.Black);
                items_item2 = help_r.GetItemsInRelationWith(item2, 0);
                items_item1 = help_r.GetItemsInRelationWith(item1, 1);
                items_item2.ForEach(item => help_r.AddItemToRelation(item1, item, Brushes.Black));
                items_item1.ForEach(item => help_r.AddItemToRelation(item, item2, Brushes.Black));
                items_item1.Clear(); items_item2.Clear();
                items_item2 = help_r.GetItemsInRelationWith(item2, 1);
                items_item1 = help_r.GetItemsInRelationWith(item1, 0);
                items_item2.ForEach(item => help_r.AddItemToRelation(item, item1, Brushes.Black));
                items_item1.ForEach(item => help_r.AddItemToRelation(item2, item, Brushes.Black));
                if (Computing.IsSymmetry(help_r) && Computing.IsTransitive(help_r)) r = help_r.Clone();
                else help_r = r.Clone();

            }
            return r;
        }

        public static List<List<int>> DecompositionEquivalenceRelation(Set set,Relation relation)
        {
            List<List<int>> classes = new List<List<int>>();
            List<int> items_set = new List<int>();
            for (int i = 0; i < set.CountItemsInSet(); i++) items_set.Add(i);

            while (items_set.Count != 0)
            {
                classes.Add(relation.GetItemsInRelationWith(items_set[0], 0));
                classes.Last().ForEach(item => items_set.Remove(item));
            }
            return classes;
        }

        //Coffman-Graham algorithm
        public static List<object> CoffmanGrahamAlgorithm(Set set, Relation order_relation, int w)
        {
            Relation c_relation =  CoverageRelation(order_relation, "");
            return new List<object>() { ReductionCrossing(SetLevels(c_relation, set, NumberingItems(c_relation, set), w), set, c_relation), 
                                        c_relation }; 
        }

        static int[] NumberingItems(Relation c_relation, Set set)
        {
            int[] index = new int[set.CountItemsInSet()];
            List<int> s2 = new List<int>();
            List<int> s1 = MinimalItem1(c_relation, set);
            int counter = 0;
            int counter1 = -1;
            while (true)
            {
                s1.Add(-1);
                s1.ForEach(item => { if (item != -1) { index[item] = counter; counter++; } });

                for (int i = 0; s1[i] != -1; i++) s1.AddRange(c_relation.GetItemsInRelationWith(s1[i], 0));
                s1.RemoveRange(0, s1.FindIndex(item => item == -1) + 1);
                if (s1.Count == 0) break;
                for (int i = 0; i < s1.Count; i++)
                {
                    for (int j = i; j < s1.Count; j++) if (s1[i] == s1[j] && i != j) { s1.RemoveAt(j); j--; }
                }
                for (int i = 0; i < s1.Count; i++) s2.Add(c_relation.GetItemsInRelationWith(s1[i], 1).Max(item => index[item]));
                s1 = s1.OrderBy(item => s2[++counter1]).ToList();
                counter1 = -1;
                s2.Clear();
            }
            return index;
        }

        static List<List<int>> SetLevels(Relation c_relation, Set set, int[] index, int w)
        {
            List<List<int>> levels = new List<List<int>>(); levels.Add(new List<int>());
            List<int> items;
            List<int> disoblige =  new List<int>();
            items = MaximalItem1(c_relation, set);
            while (true)
            {
                items = items.OrderByDescending(item => index[item]).ToList();
                if(items.Count == 0) levels.Add(new List<int>());
                items.ForEach(item =>
                {
                    if (levels.Last().Count < w) levels.Last().Add(item);
                    else
                    {
                        levels.Add(new List<int>());
                        levels.Last().Add(item);
                    }
                });
                items.Add(-1);
                for (int i = 0; items[i] != -1; i++) items.AddRange(c_relation.GetItemsInRelationWith(items[i], 1));
                items.AddRange(disoblige);
                disoblige.Clear();
                items.RemoveRange(0, items.FindIndex(item => item == -1) + 1);
                if (items.Count == 0) break;
                for (int i = 0; i < items.Count; i++)
                {
                    for (int j = i; j < items.Count; j++) if (items[i] == items[j] && i != j) { items.RemoveAt(j); j--; }
                }
                for (int i = 0; i < items.Count; i++)
                {
                    if (!c_relation.GetItemsInRelationWith(items[i], 0).All(item => levels.Exists(level => 
                    {
                        if (level.Equals(levels.Last()) && level.Count < w) return false;//!level.Exists(item1 => item1 == item);
                        else return level.Exists(item1 => item1 == item);
                    })))
                    {
                        disoblige.Add(items[i]);
                        items.RemoveAt(i); i--;
                    }
                }
            }
            levels.Reverse();
            return levels;
        }

        static Dictionary<int,PointF> ReductionCrossing(List<List<int>> levels, Set set, Relation c_relation)
        {
            int counter = 1, level; int counter1 = set.CountItemsInSet();
            int l_f_item, l_s_item, f_item, s_item;
            float p_f_item, p_s_item;
            float p_item; int found_item; int index_found_item; float help_item;
            Dictionary<int, PointF> real_position_items = new Dictionary<int, PointF>();
            real_position_items.Add(-1, new PointF(levels.Max(item => item.Count()), levels.Count));

            for (int i = 0; i < levels.Count; i++) for (int j = 0; j < levels[i].Count; j++) real_position_items.Add(levels[i][j], new PointF(j, i));

            for (int i = 0; i < levels.Count; i++) LocateItems(real_position_items, levels[i], i);

            for (int i = 0; i < c_relation.CountItemsInRelation(); i++)
            {
                f_item = c_relation.GetItemOfItemOfRelation(i, 0);
                s_item = c_relation.GetItemOfItemOfRelation(i, 1);
                l_f_item = levels.FindIndex(level1 => level1.Exists(item1 => item1 == f_item));
                l_s_item = levels.FindIndex(level1 => level1.Exists(item1 => item1 == s_item));

                if (l_s_item - l_f_item > 1)
                {
                    c_relation.RemoveItemFromRelation(f_item, s_item); i--;
                    p_f_item = real_position_items[f_item].X;
                    p_s_item = real_position_items[s_item].X;
                    level = l_f_item + counter;
                    while (level != l_s_item)
                    {
                        p_item = (float)(level - l_f_item) / (l_s_item - l_f_item) * (p_s_item - p_f_item) + p_f_item;
                        if (levels[level].Exists(item => real_position_items[item].X == p_item))
                        {
                            found_item = levels[level].Find(item => real_position_items[item].X == p_item);
                            index_found_item = levels[level].IndexOf(found_item);
                            if (p_item <= real_position_items[s_item].X)
                            {
                                if (index_found_item == levels[level].Count - 1) p_item += 0.5f;
                                else
                                {
                                    help_item = (real_position_items[levels[level][index_found_item + 1]].X - p_item) / 2;
                                    if (help_item > 0.5f) p_item += 0.5f;
                                    else p_item += help_item;
                                }
                            }
                            else
                            {
                                if (index_found_item == 0) p_item -= 0.5f;
                                else
                                {
                                    help_item = (p_item - real_position_items[levels[level][index_found_item - 1]].X) / 2;
                                    if (help_item > 0.5f) p_item -= 0.5f;
                                    else p_item -= help_item;
                                }
                            }
                        }
                        real_position_items.Add(counter1, new PointF(p_item, level));
                        levels[level].Add(counter1); levels[level] = levels[level].OrderBy(item => real_position_items[item].X).ToList();
                        c_relation.AddItemToRelation(f_item, counter1, Brushes.Black);
                        counter++;
                        level = l_f_item + counter;
                        if (level == l_s_item) c_relation.AddItemToRelation(counter1, s_item, Brushes.Black);
                        f_item = counter1;
                        counter1++;
                    }
                    counter = 1;
                }
            }
            real_position_items[-1] = new PointF(levels.Max(item => item.Count()), levels.Count);

            for (int i = 0; i < levels.Count; i++) for (int j = 0; j < levels[i].Count; j++) real_position_items[levels[i][j]] = new PointF(j, i);

            for (int i = 0; i < levels.Count; i++) LocateItems(real_position_items, levels[i], i);

            List<int> actual_level; bool change = true; Point counter_crossing; float pom; PointF p_pom;
            for (int j = 1; j < levels.Count; j++)
            {
                actual_level = levels[j];
                if (actual_level.Count > 1)
                {
                    while (change == true)
                    {
                        change = false;
                        for (int i = 0; i < actual_level.Count - 1; i++)
                        {
                            counter_crossing = CountCrossing(c_relation, actual_level, levels[j - 1], actual_level[i], actual_level[i + 1]);
                            if (counter_crossing.X > counter_crossing.Y)
                            {
                                change = true;
                                pom = actual_level[i]; actual_level[i] = actual_level[i + 1]; actual_level[i + 1] = (int)pom;
                                p_pom = real_position_items[actual_level[i]];
                                real_position_items[actual_level[i]] = real_position_items[actual_level[i + 1]];
                                real_position_items[actual_level[i + 1]] = p_pom;
                            }
                        }
                    }
                    change = true;
                }
            }

            change = true;
            for (int j = levels.Count - 2; j >= 0; j--)
            {
                actual_level = levels[j];
                if (actual_level.Count > 1)
                {
                    while (change == true)
                    {
                        change = false;
                        for (int i = 0; i < actual_level.Count - 1; i++)
                        {
                            counter_crossing = CountCrossing(c_relation, actual_level, levels[j + 1], actual_level[i], actual_level[i + 1]);
                            if (counter_crossing.X > counter_crossing.Y)
                            {
                                change = true;
                                pom = actual_level[i]; actual_level[i] = actual_level[i + 1]; actual_level[i + 1] = (int)pom;
                                p_pom = real_position_items[actual_level[i]];
                                real_position_items[actual_level[i]] = real_position_items[actual_level[i + 1]];
                                real_position_items[actual_level[i + 1]] = p_pom;
                            }
                        }
                    }
                    change = true;
                }
            }
            return real_position_items;
        }

        static void LocateItems(Dictionary<int, PointF> real_position_items, List<int> level,int number_of_level)
        {
            if (level.Count == 1) real_position_items[level[0]] = new PointF((real_position_items[-1].X - 1) / 2, number_of_level);
            else for (int j = 0; j < level.Count; j++) 
                real_position_items[level[j]] = new PointF((float)j * (real_position_items[-1].X - 1) / ((float)level.Count - 1), number_of_level);
        }

        static Point CountCrossing(Relation r, List<int> actual_level, List<int> ordered_level, int u, int v)
        {
            Point p = new Point(0, 0);
            List<int> n_u = ordered_level.Intersect(r.GetItemsInRelationWith(u, 1)).ToList().ConvertAll(item => ordered_level.IndexOf(item));
            List<int> n_v = ordered_level.Intersect(r.GetItemsInRelationWith(v, 1)).ToList().ConvertAll(item => ordered_level.IndexOf(item));
            for (int i = 0; i < n_u.Count; i++) for (int j = 0; j < n_v.Count; j++) if (n_u[i] > n_v[j]) p.X++; else if (n_u[i] < n_v[j]) p.Y++;

            return p;
        }
    }
}
