﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace LibBinaryRelation.Controls
{
    public partial class BinaryRelationOnSet : UserControl
    {
        public BinaryRelationOnSet()
        {
            InitializeComponent();

            items = new List<string>();
            items.Add("a");
            items.Add("b");
            items.Add("c");
            R1 = new Relation("K");
            R2 = new Relation("R");
            set = new Set("X");
            items.ForEach(item => set.AddItemToSet(item));
            this.selectorPair1.AddItems(items, items);

            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 3; j++)
                {
                    R1.AddItemToRelation(i, j, Brushes.Black);
                }
            }

            signed_items = new List<int>();
            c_s = new CreateSet();
            c_s.SetMaxCountItems = 20;
            c_s.SetLengthItem = 10;
        }

        Relation R1;
        Relation R2;
        Set set;
        List<int> signed_items;
        CreateSet c_s;
        List<string> items;

        private void CartesianProductPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.SignItemOfRelation(p, g, rect, R1, set, set, this.Font, signed_items, Pens.Black);
            Drawing1.DrawRelation(p, g, rect, set, set, R1, this.Font);
        }

        private void RelationPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, R2, this.Font);
        }

        public Font GetMaxFont()
        {
            Button b = this.AddItem;
            return Drawing1.GetFont(b.CreateGraphics(), b.Width, b.Height, "\t" + b.Text + "\t", b.Font);
        }

        private void AddItemClick(object sender, EventArgs e)
        {
            this.RemoveItem.Enabled = true;
            Point items = this.selectorPair1.GetCurrentItems;
            if (items.X != -1 && R2.GetNumberOfItemOfRelation(items.X, items.Y) == -1)
            {
                R2.AddItemToRelation(items.X, items.Y, Brushes.Black);
                signed_items.Add(R1.GetNumberOfItemOfRelation(items.X, items.Y));
                if (signed_items.Count == R1.CountItemsInRelation()) this.AddItem.Enabled = false;
                Relation.Invalidate();
                CartesianProduct.Invalidate();
            }
            else this.selectorPair1.Signal();

            SetButtons();
        }

        private void RemoveItemClick(object sender, EventArgs e)
        {
            this.AddItem.Enabled = true;
            Point items = this.selectorPair1.GetCurrentItems;
            if (R2.RemoveItemFromRelation(items.X, items.Y) == false) this.selectorPair1.Signal();
            signed_items.Remove(R1.GetNumberOfItemOfRelation(items.X, items.Y));
            if (signed_items.Count == 0) this.RemoveItem.Enabled = false;
            Relation.Invalidate();
            CartesianProduct.Invalidate();

            SetButtons();
        }

        void SetButtons()
        {
            bool h;
            if (R2.CountItemsInRelation() == 0) h = false;
            else h = true;
            this.RemoveItem.Enabled = h;

            if (R2.CountItemsInRelation() >= set.CountItemsInSet() * set.CountItemsInSet()) this.AddItem.Enabled = false;
            else this.AddItem.Enabled = true;
        }

        private void BinaryRelationOnSetResize(object sender, EventArgs e)
        {
            this.Relation.Invalidate();
            this.CartesianProduct.Invalidate();
            this.DrawSet.Invalidate();
        }

        private void panel1Paint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width - v_space, p.Height), set, this.Font);
        }

        private void CreateSetClick(object sender, EventArgs e)
        {
            c_s.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_s, this.BackColor);
            c_s.Set = set;
            DialogResult d = c_s.ShowDialog();
            if (d == DialogResult.OK)
            {
                set = c_s.Set;
                R1.RemoveAllItems();
                R2.RemoveAllItems();
                items.Clear();
                signed_items.Clear();
                for (int i = 0; i < set.CountItemsInSet(); i++)
                {
                    items.Add(set.GetNameItem(i));
                    for (int j = 0; j < set.CountItemsInSet(); j++)
                    {
                        R1.AddItemToRelation(i, j, Brushes.Black);
                    }
                }
                if (set.CountItemsInSet() == 0) this.ArrangementR.Enabled = false;
                else this.ArrangementR.Enabled = true;
                this.AddItem.Enabled = this.AddItem.Enabled = true;
                this.selectorPair1.AddItems(items, items);
                this.Relation.Invalidate();
                this.CartesianProduct.Invalidate();
                this.DrawSet.Invalidate();
            }
        }

        public virtual Size MyMinimumSize()
        {
            Graphics g = this.CreateGraphics();
            int width = (int)g.MeasureString(this.DescriptionCP.Text + this.AddItem.Text + this.DescriptionR.Text, this.Font).Width;
            return new Size(width, width);
        }
    }
}
