﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class CartesianGraphRelation : UserControl
    {
        public CartesianGraphRelation()
        {
            InitializeComponent();

            set1 = new Set("X");
            set2 = new Set("Y");
            relation = new Relation("R");
            signed_items = new List<int>() {0};

            set1.AddItemToSet("-3"); set1.AddItemToSet("-2"); set1.AddItemToSet("-1");
            set1.AddItemToSet("0"); set1.AddItemToSet("1"); set1.AddItemToSet("2"); set1.AddItemToSet("3");
            set2.AddItemToSet("9"); set2.AddItemToSet("4"); set2.AddItemToSet("1"); set2.AddItemToSet("0");
            relation.AddItemToRelation(0, 0, Brushes.Black);
            relation.AddItemToRelation(1, 1, Brushes.Black);
            relation.AddItemToRelation(2, 2, Brushes.Black);
            relation.AddItemToRelation(3, 3, Brushes.Black);
            relation.AddItemToRelation(4, 2, Brushes.Black);
            relation.AddItemToRelation(5, 1, Brushes.Black);
            relation.AddItemToRelation(6, 0, Brushes.Black);
        }

        Relation relation;
        Set set1;
        Set set2;
        List<int> signed_items;

        private void RelationPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.SignItemOfRelation(p, g, rect, relation, set1, set2, this.Font, signed_items, Pens.Black);
            Drawing1.DrawRelation(p, g, rect, set1, set2, relation, this.Font);
        }

        private void MoveLeftClick(object sender, EventArgs e)
        {
            this.MoveRight.Enabled = true;
            if (signed_items[0] == 1) this.MoveLeft.Enabled = false;
            signed_items.Add(signed_items[0]--);
            signed_items.RemoveAt(1);
            this.Relation.Invalidate();
            this.CartGraph.Invalidate();
        }

        private void MoveRightClick(object sender, EventArgs e)
        {
            this.MoveLeft.Enabled = true;
            if (signed_items[0] == this.set1.CountItemsInSet() - 2) this.MoveRight.Enabled = false;
            signed_items.Add(signed_items[0]++);
            signed_items.RemoveAt(1);
            this.Relation.Invalidate();
            this.CartGraph.Invalidate();
        }

        private void CartGraphPaint(object sender, PaintEventArgs e)
        {
            PictureBox p = (PictureBox)sender;
            Graphics g = e.Graphics;
            g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            float ten_percent_from_height = p.Height * 0.1f;
            float height = p.Height - 2 * ten_percent_from_height;
            Font font = new Font(this.Font.FontFamily, ten_percent_from_height / 2, this.Font.Style, GraphicsUnit.Pixel);
            float one_item_y = height * 0.1f;
            float one_item_x = height * 0.2f;
            float width_line = one_item_y * 0.5f;
            RectangleF rect = new RectangleF(p.Width/2 - (3 * one_item_x + one_item_y), ten_percent_from_height / 2,6 * one_item_x + 2 * one_item_y,height);
            
            //
            float x = rect.X + rect.Width / 2;
            float y = rect.Y + rect.Height;
            float first_x = x;
            float first_y = y;
            float average_of_point = one_item_y * 0.4f;
            float radius_of_point = average_of_point / 2;
            PointF[] points_of_function = new PointF[7];
            SizeF size_item = g.MeasureString("-1", font);//
            RectangleF rect_item = new RectangleF(0, 0, size_item.Width, size_item.Height);//
            StringFormat s = new StringFormat(); s.Alignment = StringAlignment.Center;//

            for (int i = 0; i < 4; i++)
            {
                first_y = y - (float)Math.Pow(i, 2) * one_item_y;

                g.DrawLine(Pens.Gray, first_x, y, first_x, first_y);
                g.DrawLine(Pens.Gray, first_x, first_y, x, first_y);

                rect_item.X = first_x - rect_item.Width / 2;
                rect_item.Y = y + width_line / 2 + ten_percent_from_height / 4;
                g.DrawLine(Pens.Black, first_x, y - width_line / 2, first_x, y + width_line/2);
                g.DrawString(set1.GetNameItem(3 - i), font, Brushes.Black, rect_item, s);
                if (i > 0)
                {
                    g.DrawLine(Pens.Black, x - width_line/2, first_y, x + width_line/2, first_y);
                    rect_item.X = x - rect_item.Width - width_line / 2;
                    rect_item.Y = first_y - rect_item.Height / 2;
                    g.FillRectangle(Brushes.Transparent, rect_item);
                    g.DrawString(set2.GetNameItem(3 - i), font, Brushes.Black, rect_item, s);
                }

                points_of_function[3 - i] = new PointF(first_x, first_y);
                Drawing1.DrawItemOfSet(g, Brushes.Black, average_of_point, first_x - radius_of_point, first_y - radius_of_point);
                first_x -= one_item_x;
            }

            //
            first_x = x + one_item_x;
            for (int i = 1; i < 4; i++)
            {
                first_y = y - (float)Math.Pow(i, 2) * one_item_y;
                g.DrawLine(Pens.Gray, first_x, y, first_x, first_y);
                g.DrawLine(Pens.Gray, first_x, first_y, x, first_y);

                rect_item.X = first_x - rect_item.Width / 2;
                rect_item.Y = y + width_line / 2 + ten_percent_from_height / 4;
                g.DrawLine(Pens.Black, first_x, y - width_line / 2, first_x, y + width_line / 2);
                g.DrawString(set1.GetNameItem(i + 3), font, Brushes.Black, rect_item, s);

                points_of_function[i + 3] = new PointF(first_x, first_y);
                Drawing1.DrawItemOfSet(g, Brushes.Black, average_of_point, first_x - radius_of_point, first_y - radius_of_point);
                first_x += one_item_x;
            }

            //
            Drawing1.DrawArrow(g, new PointF(rect.X + rect.Width / 2, rect.Y + rect.Height), new PointF(rect.X + rect.Width / 2, rect.Y), one_item_y * 0.5f, 1, Brushes.Black, true, true);
            Drawing1.DrawArrow(g, new PointF(rect.X, rect.Y + rect.Height), new PointF(rect.X + rect.Width, rect.Y + rect.Height), width_line, 1, Brushes.Black, true, true);
            Drawing1.DrawArrow(g, new PointF(rect.X + rect.Width, rect.Y + rect.Height), new PointF(rect.X, rect.Y + rect.Height), width_line, 1, Brushes.Black, true, false);
            g.DrawCurve(Pens.Black, points_of_function);
            
            //
            Drawing1.DrawItemOfSet(g, Brushes.Red, 2 * average_of_point, points_of_function[signed_items[0]].X - average_of_point, points_of_function[signed_items[0]].Y - average_of_point);
            
            //
            g.DrawString("x", font, Brushes.Black, rect.X + rect.Width - size_item.Width, y - size_item.Height - width_line / 2);
            g.DrawString("y", font, Brushes.Black, x + width_line / 2, rect.Y );
        }

        public Font GetMaxFont()
        {
            Label l = this.DescriptionSet1;
            Graphics g = l.CreateGraphics();
            Font f = Drawing1.GetFont(g, l.Width, l.Height, "M" + l.Text + "M", this.Font);
            return f;
        }

        private void CartesianGraphRelationResize(object sender, EventArgs e)
        {
            CartGraph.Invalidate();
            Relation.Invalidate();
        }

        public virtual Size MyMinimumSize()
        {
            Graphics g = this.CreateGraphics();
            float width = g.MeasureString(this.DescriptionSet1.Text,this.Font).Width + g.MeasureString(this.DescriptionSet1.Text, this.Font).Width ;
            return new Size((int)width, (int)(width * 0.75f));
        }
    }
}
