﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class Closures : UserControl
    {
        public Closures()
        {
            InitializeComponent();

            List<string> items = new List<string>() { "a", "b", "c", "d", "e" };

            this.selectorPair1.AddItems(items, items);

            set = new Set("X");
            items.ForEach(item => set.AddItemToSet(item));

            relation = new Relation("L");

            reflexive_c_relation = Computing.ReflexiveCRelation(relation, set, "R");

            symmetry_c_relation = new Relation("S");
            transitive_c_relation = new Relation("T");

            c_set = new CreateSet();
            c_set.SetMaxCountItems = 20;
            c_set.SetLengthItem = 10;
        }

        Relation relation;
        Relation reflexive_c_relation;
        Relation symmetry_c_relation;
        Relation transitive_c_relation;
        Set set;
        CreateSet c_set;

        public Font GetMaxFont()
        {
            GroupBox l = this.DescriptionR;
            Graphics g = l.CreateGraphics();
            return Drawing1.GetFont(g, l.Width, l.Height, "\t" + l.Text + "\t", l.Font);
        }

        private void DrawRelationLPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, relation, this.Font);
        }

        private void DrawRelationRPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, reflexive_c_relation, this.Font);
        }

        private void DrawRelationSPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, symmetry_c_relation, this.Font);
        }

        private void DrawRelationTPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, transitive_c_relation, this.Font);
        }

        private void AddItemClick(object sender, EventArgs e)
        {
           Point current_items = this.selectorPair1.GetCurrentItems;
           if (current_items.X != -1 && current_items.Y != -1)
           {
               if (relation.AddItemToRelation(current_items.X, current_items.Y, Brushes.Black) == false) this.selectorPair1.Signal();
               else
               {
                   SetAllRelations();
                   this.DrawRelationL.Invalidate();
                   this.DrawRelationR.Invalidate();
                   this.DrawRelationS.Invalidate();
                   this.DrawRelationT.Invalidate();
               }
           }

           SetButtons();
        }

        private void RemoveItemClick(object sender, EventArgs e)
        {
           Point current_items = this.selectorPair1.GetCurrentItems;
           if (current_items.X != -1 && current_items.Y != -1)
           {
               if (relation.RemoveItemFromRelation(current_items.X, current_items.Y) == false) this.selectorPair1.Signal();
               else
               {
                   SetAllRelations();
                   this.DrawRelationL.Invalidate();
                   this.DrawRelationR.Invalidate();
                   this.DrawRelationS.Invalidate();
                   this.DrawRelationT.Invalidate();
               }
           }

           SetButtons();
        }

        void SetButtons()
        {
            bool h;
            if (relation.CountItemsInRelation() == 0) h = false;
            else h = true;
            this.RemoveItem.Enabled = h;

            if (relation.CountItemsInRelation() >= set.CountItemsInSet() * set.CountItemsInSet()) this.AddItem.Enabled = false;
            else this.AddItem.Enabled = true;
        }

        void SetAllRelations()
        {
            reflexive_c_relation = Computing.ReflexiveCRelation(relation, set, "R");
            symmetry_c_relation = Computing.SymmetryCRelation(relation, "S");
            transitive_c_relation = Computing.TransitiveCRelation(relation, set, "T");
        }

        /*private void DrawRelationLResize(object sender, EventArgs e)
        {
            this.DrawRelationL.Invalidate();
            this.DrawRelationR.Invalidate();
            this.DrawRelationS.Invalidate();
            this.DrawRelationT.Invalidate();
            this.DrawSet.Invalidate();
        }*/

        private void DrawSetPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width, p.Height), set, this.Font);
        }

        private void CreateSetClick(object sender, EventArgs e)
        {
            c_set.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_set, this.BackColor);
            c_set.Set = set;
            DialogResult d = c_set.ShowDialog();
            if (d == DialogResult.OK)
            {
                set = c_set.Set;
                List<string> l = new List<string>();
                for (int i = 0; i < set.CountItemsInSet(); i++) l.Add(set.GetNameItem(i));
                this.selectorPair1.AddItems(l, l);
                symmetry_c_relation.RemoveAllItems();
                transitive_c_relation.RemoveAllItems();
                relation.RemoveAllItems();
                SetButtons();
                reflexive_c_relation = Computing.ReflexiveCRelation(relation, set, "R");
                this.DrawRelationL.Invalidate();
                this.DrawRelationR.Invalidate();
                this.DrawRelationS.Invalidate();
                this.DrawRelationT.Invalidate();
                this.DrawSet.Invalidate();
                if (set.CountItemsInSet() == 0) groupBox1.Enabled = false;
                else groupBox1.Enabled = true;
            }
        }

        private void ClosuresResize(object sender, EventArgs e)
        {
            this.DrawRelationL.Invalidate();
            this.DrawRelationR.Invalidate();
            this.DrawRelationS.Invalidate();
            this.DrawRelationT.Invalidate();
            this.DrawSet.Invalidate();
        }

        public virtual Size MyMinimumSize()
        {
            Graphics g = this.CreateGraphics();
            float width = 4 * g.MeasureString(this.DescriptionR.Text, this.Font).Width;
            return new Size((int)width, (int)(width * 0.75f));
        }
    }
}
