﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class Coverage : UserControl
    {
        public Coverage()
        {
            InitializeComponent();

            List<string> items = new List<string>() { "1", "2", "3", "4", "5", "6", };

            set = new Set("X");
            items.ForEach(item => set.AddItemToSet(item));

            r1 = new Relation("R");

            for (int i = 0; i < set.CountItemsInSet(); i++)
            {
                for (int j = i; j < set.CountItemsInSet(); j++) r1.AddItemToRelation(i, j, Brushes.Black);
            }

            r2 = Computing.CoverageRelation(r1, "L");

            signed_item = new List<int>() { -1, -1, -1 };
            signed_item1 = new List<int>() { -1 };
            actual_item = 0;
            SetItems();

            d = new Dictionary<int, int>();
            d.Add(2, 1); d.Add(3, 2); d.Add(4, 2); d.Add(5, 3);
            d.Add(8, 2); d.Add(9, 3); d.Add(10, 2); d.Add(13, 3);
            d.Add(14, 3); d.Add(17, 4);
        }

        Relation r1;
        Relation r2;
        Set set;
        List<int> signed_item;
        List<int> signed_item1;
        int actual_item;
        Dictionary<int, int> d;

        private void DrawRelationRPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.SignItemOfRelation(p, g, rect, r1, set, set, this.Font, signed_item, Pens.Red, Pens.Black, Pens.Black);
            Drawing1.DrawRelation(p, g, rect, set, set, r1, p.Font);
        }

        private void DrawRelationLPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.SignItemOfRelation(p, g, rect, r2, set, set, this.Font, signed_item1, Pens.Red);
            Drawing1.DrawRelation(p, g, rect, set, set, r2, p.Font);
        }

        public Font GetMaxFont()
        {
            GroupBox l = this.CaptionR;
            Graphics g = l.CreateGraphics();
            return Drawing1.GetFont(g, l.Width, l.Height, "\t" + l.Text + "\t", l.Font);
        }

        private void MoveRightClick(object sender, EventArgs e)
        {
            MoveLeft.Enabled = true;
            actual_item++;
            if (actual_item >= r1.CountItemsInRelation() - 1) MoveRight.Enabled = false;
            SetItems(); 
        }

        private void MoveLeftClick(object sender, EventArgs e)
        {
            MoveRight.Enabled = true;
            actual_item--;
            if (actual_item < 1) MoveLeft.Enabled = false;
            SetItems();
        }

        void DrawText(int position, int first_item, int second_item)
        {
            int f_i = first_item + 1;
            int s_i = second_item + 1;
            if (position == -1)
            {
                this.Caption1.Text = "Uspořádaná dvojice prvků " + f_i + ", " + s_i + " neleží v relaci L";

                if (first_item == second_item) this.Caption2.Text = "Důvod: relace L je irreflexivní";
                else this.Caption2.Text = "Důvod: Existují v relaci R uspořádané dvojice prvků " + f_i + ",y a y," + s_i + ", kde y je prvek z množiny X a je různý od prvků " + f_i + ", " + s_i;
            }
            else
            {
                this.Caption1.Text = "Uspořádaná dvojice prvků " + f_i + ", " + s_i + " leží v relaci L";
                this.Caption2.Text = "Důvod: Neexistují v relaci R uspořádané dvojice prvků " + f_i + ",y a y," + s_i + ", kde y je prvek z množiny X a je různý od prvků " + f_i + ", " + s_i; 
            }

        }

        void SetItems()
        {
            int first_item = r1.GetItemOfItemOfRelation(actual_item, 0);
            int second_item = r1.GetItemOfItemOfRelation(actual_item, 1);
            int position_item = r2.GetNumberOfItemOfRelation(first_item, second_item);
            DrawText(position_item, first_item, second_item);
            signed_item[0] = actual_item;
            if (position_item == -1)
            {
                signed_item1[0] = - 1;
                if (first_item != second_item)
                {
                    int item = d[actual_item];
                    signed_item[1] = r1.GetNumberOfItemOfRelation(first_item, item);
                    signed_item[2] = r1.GetNumberOfItemOfRelation(item, second_item);
                }
                else
                {
                    signed_item[1] = -1;
                    signed_item[2] = -1;
                }
            }
            else
            {
                signed_item[1] = -1;
                signed_item[2] = -1;
                signed_item1[0] = position_item;
            }
            this.DrawRelationL.Invalidate();
            this.DrawRelationR.Invalidate();
        }

        private void CoverageResize(object sender, EventArgs e)
        {
            this.DrawRelationL.Invalidate();
            this.DrawRelationR.Invalidate();
        }
    }
}
