﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class Diagram : UserControl
    {
        public Diagram()
        {
            InitializeComponent();
            number_horizontal_lines = number_vertical_lines = 0;
            width_square = height_square = 0;
            names_items = new List<string>();
            position_items = new List<float>();
            arrows = new List<int>();
            signs = new List<int>();
            colors = new List<Pen>();
            g = this.Surface.CreateGraphics();
            index_of_item = -1;
            width_item = -1;
        }

        int number_horizontal_lines;
        int number_vertical_lines;
        float width_square;
        float height_square;
        List<string> names_items;
        List<float> position_items;
        List<int> arrows;
        List<int> signs;
        List<Pen> colors;
        Graphics g;
        float width_item;
        int index_of_item;

        public enum SizeSign { Small, Medium, Large };

        public void SetGrid(int count_horizontal_lines, int count_vertical_lines)
        {
            this.number_horizontal_lines = count_horizontal_lines;
            this.number_vertical_lines = count_vertical_lines;
        }

        public void SignsClear()
        {
            signs.Clear(); colors.Clear();
        }

        public void Clear()
        {
            names_items.Clear(); position_items.Clear(); arrows.Clear(); signs.Clear(); colors.Clear();
            number_horizontal_lines = number_vertical_lines = 0;
            width_square = height_square = 0; index_of_item = -1; width_item = -1;

        }

        public void AddNode(string name_node, float x_position, float y_position, bool node_is_visible)
        {
            names_items.Add(name_node);
            position_items.Add(x_position + 1); position_items.Add(y_position + 1); position_items.Add(node_is_visible ? 1 : 0);
            float width_item1 = g.MeasureString(name_node, this.Font).Width;
            if (width_item1 > width_item) { index_of_item = names_items.Count - 1; width_item = width_item1; }
        }

        public bool AddArrow(string name_node1, string name_node2)
        {
            int first_item = names_items.FindIndex(item => name_node1 == item);
            if (first_item == -1) return false;

            int second_item = names_items.FindIndex(item => name_node2 == item);
            if (second_item == -1) return false;

            arrows.Add(first_item); arrows.Add(second_item);
            return true;
        }

        void SetSizeSquare()
        {
            if (number_horizontal_lines != 0 && number_vertical_lines != 0)
            {
                width_square = Surface.Width / (number_vertical_lines + 1);
                height_square = Surface.Height / (number_horizontal_lines + 1);
            }
        }

        void DrawNodes(Graphics g, float average_of_item)
        {
            float radius = average_of_item / 2;

            StringFormat s = new StringFormat(); s.Alignment = s.LineAlignment = StringAlignment.Center;
            RectangleF rect = new RectangleF();
            for (int i = 0; i < names_items.Count; i++)
            {
                if (position_items[3 * i + 2] == 1)
                {
                    float x = position_items[3 * i] * width_square;
                    float y = position_items[3 * i + 1] * height_square;
                    g.FillEllipse(Brushes.Black, x - radius, y - radius, average_of_item, average_of_item);
                    SizeF size = g.MeasureString(names_items[i], this.Font);
                    rect.X = x + radius + radius / 2; rect.Y = y - size.Height / 2; rect.Width = size.Width; rect.Height = size.Height;
                    g.FillRectangle(Brushes.LightGray, rect);
                    g.DrawString(names_items[i], this.Font, Brushes.Black, rect, s);
                }
            }

        }

        void DrawSigns(Graphics g, float average_of_item)
        {
            int index;
            float average_of_item1, radius;
            for (int i = 0; i < signs.Count; i += 3)
            {
                index = signs[i];
                float x = position_items[3 * index] * width_square;
                float y = position_items[3 * index + 1] * height_square;
                index = signs[i + 2];
                average_of_item1 = index == (int)SizeSign.Small ? 10 : index == (int)SizeSign.Medium ? 20 : 30;
                average_of_item1 += average_of_item;
                radius = average_of_item1 / 2;
                g.DrawEllipse(colors[signs[i + 1]], x - radius, y - radius, average_of_item1, average_of_item1);
            }
        }

        void DrawArrows(Graphics g, float average_of_item)
        {
            PointF p1 = new PointF();
            PointF p2 = new PointF();
            int f_item, s_item;
            for (int i = 0; i < arrows.Count; i += 2)
            {
                f_item = arrows[i]; s_item = arrows[i + 1];
                p1.X = position_items[3 * f_item] * width_square;
                p1.Y = position_items[3 * f_item + 1] * height_square;
                p2.X = position_items[3 * s_item] * width_square;
                p2.Y = position_items[3 * s_item + 1] * height_square;
                g.DrawLine(Pens.Black, p2, p1);
            }
        }

        public void DiagramPaint()
        {
            this.Surface.Invalidate();
        }

        public void SetSign(string name_item, Pen color, SizeSign size_sign)
        {
            signs.Add(names_items.FindIndex(item => name_item == item));
            colors.Add(color);
            signs.Add(colors.Count - 1);
            signs.Add((int)size_sign);
        }

        public void SetAsHasseDiagram(Set set, Relation order_relation, int w)
        {
            this.Clear();
            List<object> c_g_a = Computing.CoffmanGrahamAlgorithm(set, order_relation, w);

            Dictionary<int, PointF> c_g_a_p_i = (Dictionary<int, PointF>)c_g_a[0];
            Relation c_g_a_c_r = (Relation)c_g_a[1];

            float count_levels = c_g_a_p_i[-1].Y;
            this.SetGrid((int)count_levels, (int)c_g_a_p_i[-1].X);
            count_levels--;
            int index, index1;
            string model = "~(/";
            Dictionary<int, string> d = new Dictionary<int, string>();
            int count_items = set.CountItemsInSet();

            for (int i = 0; i < count_items; i++) this.AddNode(set.GetNameItem(i), c_g_a_p_i[i].X, count_levels - c_g_a_p_i[i].Y, true);
            for (int i = set.CountItemsInSet(); i < c_g_a_p_i.Count - 1; i++) { d.Add(i, model + i); this.AddNode(model + i, c_g_a_p_i[i].X, count_levels - c_g_a_p_i[i].Y, false); }
            for (int i = 0; i < c_g_a_c_r.CountItemsInRelation(); i++)
            {
                index = c_g_a_c_r.GetItemOfItemOfRelation(i, 0);
                index1 = c_g_a_c_r.GetItemOfItemOfRelation(i, 1);
                this.AddArrow(index > count_items - 1 ? d[index] : set.GetNameItem(index), index1 > count_items - 1 ? d[index1] : set.GetNameItem(index1));
            }
        }

        private void DiagramResize(object sender, EventArgs e)
        {
            this.Surface.Invalidate();
        }

        private void SurfacePaint(object sender, PaintEventArgs e)
        {
            if (names_items.Count != 0)
            {
                Graphics g = e.Graphics;
                float average_of_item = Drawing1.GetAverageOfItem(this.Font);
                float width = width_item + average_of_item * 0.5f;
                if (width < 30 + average_of_item) width = 30 + average_of_item;
                else width = width + average_of_item;
                this.AutoScrollMinSize = new Size((int)(width * (number_vertical_lines + 2)), (int)(width * (number_horizontal_lines + 1)));
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                SetSizeSquare(); ;
                DrawSigns(g, average_of_item);
                DrawArrows(g, average_of_item);
                DrawNodes(g, average_of_item);
            }
            else this.AutoScrollMinSize = new Size(0, 0);
        }

        private void DiagramFontChanged(object sender, EventArgs e)
        {
            if(names_items.Count != 0)width_item = g.MeasureString(names_items[index_of_item], this.Font).Width;
        }
    }
}
