﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using LibBinaryRelation;

namespace LibBinaryRelation.Controls
{
    public partial class EquivalenceRelation : UserControl
    {
        public EquivalenceRelation()
        {
            InitializeComponent();
            set = new Set("X");
            equivalence_class = new Set("");
            relation = new Relation("E");
            //classes = new List<List<int>>();
            c_r = new CreateRelation();
            c_r.OrderRelation = false;
            c_s = new CreateSet();
            c_s.SetLengthItem = 6;
            c_s.SetMaxCountItems = 15;
            is_order_relation = true;

            set.AddItemToSet("Martin"); set.AddItemToSet("Petr"); set.AddItemToSet("Jirka"); set.AddItemToSet("Dana"); set.AddItemToSet("Eva");
            set.AddItemToSet("Iveta"); set.AddItemToSet("Honza"); set.AddItemToSet("Michal"); set.AddItemToSet("Martina");
            for (int i = 0; i < set.CountItemsInSet(); i++) this.listBox1.Items.Add(set.GetNameItem(i));
            relation = Computing.CreateRandomEquivalenceRelation(set, relation.NameRelation);
            classes = Computing.DecompositionEquivalenceRelation(set, relation);
            classes[0].ForEach(item => equivalence_class.AddItemToSet(set.GetNameItem(item)));
            equivalence_class.NameSet = "_[" + set.GetNameItem(classes[0][0])+"]_" + relation.NameRelation + "_";
            clas = p_i_clas = 0;
        }

        Set set;
        Set equivalence_class;
        Relation relation;
        List<List<int>> classes;
        CreateRelation c_r;
        CreateSet c_s;
        bool is_order_relation;
        int clas;
        int p_i_clas;

        private void toolStripMenuItem1Click(object sender, EventArgs e)
        {
            c_s.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_s, this.BackColor);
            c_s.Set = set;
            DialogResult d = c_s.ShowDialog();
            if (d == DialogResult.OK)
            {
                set = c_s.Set;
                if (set.CountItemsInSet() == 0) is_order_relation = true;
                else is_order_relation = false;
                relation.RemoveAllItems();
                equivalence_class.RemoveAllItems();
                equivalence_class.NameSet = "";
                classes.Clear();
                this.listBox1.Items.Clear();
                for (int i = 0; i < set.CountItemsInSet(); i++) this.listBox1.Items.Add(set.GetNameItem(i));
                this.listBox1.Enabled = false;
                DrawSet.Invalidate();
                DrawRelation.Invalidate();
                DrawClass.Invalidate();
                Surface.Invalidate();
            }
        }

        private void toolStripMenuItem2Click(object sender, EventArgs e)
        {
            c_r.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_r, this.BackColor);
            c_r.Relation1 = relation;
            c_r.Set = set;
            if (c_r.ShowDialog() == DialogResult.OK)
            {
                relation = c_r.Relation1;
                equivalence_class.RemoveAllItems();
                equivalence_class.NameSet = "";
                classes.Clear();
                if (Computing.IsReflexive(relation, set) && Computing.IsSymmetry(relation) && Computing.IsTransitive(relation))
                {
                    is_order_relation = true;
                    this.listBox1.Enabled = true;
                    if (set.CountItemsInSet() != 0)
                    {
                        this.listBox1.SelectedIndex = 0;
                    }
                    clas = p_i_clas = 0;
                    classes = Computing.DecompositionEquivalenceRelation(set, relation);
                    if (classes.Count != 0)
                    {
                        classes[0].ForEach(item => equivalence_class.AddItemToSet(set.GetNameItem(item)));
                        equivalence_class.NameSet = "_[" + set.GetNameItem(classes[0][0]) + "]_" + relation.NameRelation + "_";
                    }
                }
                else is_order_relation = this.listBox1.Enabled = false;   
                DrawRelation.Invalidate();
                DrawClass.Invalidate();
                Surface.Invalidate();
            }
        }

        private void DrawSetPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width - v_space, p.Height), set, this.Font);
        }

        private void DrawRelationPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            if (is_order_relation == true) Drawing1.DrawRelation(p, g, rect, set, set, relation, this.Font);
            else
            {
                p.AutoScrollMinSize = new Size((int)g.MeasureString("Vámi vytvořená relace není relace ekvivalence", this.Font).Width + v_space, 0);
                g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                g.DrawString("Vámi vytvořená relace není relace ekvivalence", this.Font, Brushes.Black, v_space + p.AutoScrollPosition.X, (int)(p.Height * 0.1f));
            }
        }

        private void DrawClassPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            if (is_order_relation == true) Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width - v_space, p.Height), equivalence_class, this.Font);
        }

        private void EquivalenceRelationResize(object sender, EventArgs e)
        {
            DrawSet.Invalidate();
            DrawRelation.Invalidate();
            DrawClass.Invalidate();
            Surface.Invalidate();
        }

        private void SurfacePaint(object sender, PaintEventArgs e)
        {
            if (is_order_relation == true)
            {
                Graphics g = e.Graphics;
                Panel p = (Panel)sender;
                Point p_point = p.AutoScrollPosition;
                string m_l_item = Drawing1.GetTheMostLengthItemOfSetInRelation(g, set, set, this.Font);
                SizeF item_size = g.MeasureString(m_l_item, this.Font);
                float end_of_arrow = item_size.Width + 100;
                int start_y = 0; int end_y = 0;
                int h_space = 20;
                int counter = 0;
                float y_p = 1;
                float move_y = 10 + item_size.Height + h_space;
                float length_arrow = item_size.Height / 2;
                RectangleF rect = new RectangleF(end_of_arrow + p_point.X, move_y + p_point.Y, item_size.Width, item_size.Height);
                StringFormat s = new StringFormat(); s.Alignment = StringAlignment.Center;
                int b = (int)(length_arrow + length_arrow / 2);
                if (end_of_arrow + item_size.Width + b < p.Width) g.TranslateTransform(p.Width / 2 - (end_of_arrow + item_size.Width + b) / 2, 0);
                Rectangle rect1 = new Rectangle((int)rect.X - b, (int)move_y - b + p_point.Y, (int)rect.Width + 2 * b, (int)(classes.Sum(item => item.Count) * (item_size.Height + h_space) - h_space + 2 * b));
                g.DrawRectangle(Pens.Black, rect1);
                rect1.X = 0 + p_point.X;
                g.DrawRectangle(Pens.Black, rect1);
                Brush color = Brushes.Black; Pen pen = new Pen(Brushes.Black);

                g.DrawString(set.NameSet, this.Font, Brushes.Black, 0 + p_point.X, 0 + p_point.Y);
                g.DrawString(set.NameSet + "/" + relation.NameRelation, this.Font, Brushes.Black, end_of_arrow - b + p_point.X, 0 + p_point.Y);
                for (int i = 0; i < classes.Count; i++)
                {
                    for (int j = 0; j < classes[i].Count; j++)
                    {
                        y_p = counter * (item_size.Height + h_space) + move_y + p_point.Y;
                        if (clas == i && p_i_clas == j) color = Brushes.Red;
                        else color = Brushes.Black;
                        g.DrawString(set.GetNameItem(classes[i][j]), this.Font, color, rect1.Width / 2 - g.MeasureString(set.GetNameItem(classes[i][j]), this.Font).Width / 2 + p_point.X, y_p);
                        rect.Y = y_p;
                        g.DrawString(set.GetNameItem(classes[i][j]), this.Font, Brushes.Black, rect, s);
                        y_p += length_arrow;
                        Drawing1.DrawArrow(g, new PointF(rect1.Width / 2 + g.MeasureString(set.GetNameItem(classes[i][j]), this.Font).Width / 2 + p_point.X, y_p), new PointF(end_of_arrow + p_point.X, y_p), (int)length_arrow, 1, color, true, true);
                        counter++;
                    }
                    end_y = counter - start_y;
                    if (clas == i) pen.Brush = Brushes.Red;
                    else pen.Brush = Brushes.Black;
                    g.DrawRectangle(pen, end_of_arrow + p_point.X, start_y * (item_size.Height + h_space) + move_y + p_point.Y, item_size.Width, end_y * (item_size.Height + h_space) - h_space);
                    start_y = counter;
                };
                p.AutoScrollMinSize = new Size((int)(end_of_arrow + item_size.Width) + b + 10, rect1.Y + rect1.Height - p_point.Y + 10);
            }
        }

        private void listBox1SelectedIndexChanged(object sender, EventArgs e)
        {
            int s_i = listBox1.SelectedIndex;
            bool t = false;
            for (int i = 0; i < classes.Count; i++)
            {
                for (int j = 0; j < classes[i].Count; j++)
                {
                    if (classes[i][j] == s_i)
                    {
                        clas = i; p_i_clas = j;
                        equivalence_class.RemoveAllItems();
                        equivalence_class.NameSet = "_[" + set.GetNameItem(classes[i][j]) + "]_" + relation.NameRelation + "_";
                        classes[i].ForEach(item => equivalence_class.AddItemToSet(set.GetNameItem(item)));
                        t = true; break;
                    }
                }
                if (t == true) break;
            }
            this.Surface.Invalidate(); this.DrawClass.Invalidate();
        }
    }
}
