﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    partial class Graph1 : UserControl
    {
         public Graph1()
        {
            InitializeComponent();
            nodes = new List<Brush>();
            nodes1 = new List<string>();
            arrow_between_items = new ArrayList();
            count_items = 0;
            position_nodes = new List<float>();
            horizontal_lines = 0;
            vertical_lines = 0;
            width_square = height_square = 0;
        }

        List<Brush> nodes;
        List<string> nodes1;
        ArrayList arrow_between_items;
        int count_items;
        List<float> position_nodes;
        int width_square, height_square;
        int horizontal_lines;
        int vertical_lines;

        public void SetGrid(int number_horizontal_lines, int number_vertical_lines)
        {
            nodes1.Clear(); nodes.Clear(); position_nodes.Clear(); arrow_between_items.Clear();
            count_items = 0;
            this.horizontal_lines = number_horizontal_lines;
            this.vertical_lines = number_vertical_lines;
            SetSizeSquare();
        }

        void SetSizeSquare()
        {
            if (horizontal_lines != 0 && vertical_lines != 0)
            {
                width_square = this.Width / (vertical_lines + 1);
                height_square = this.Height / (horizontal_lines + 1);
            }
        }

        private void SurfacePaint(object sender, PaintEventArgs e)
        {
            if (count_items != 0)
            {
                Graphics g = e.Graphics;
                SizeF size = g.MeasureString("M", this.Font);
                float average_node = (float)Math.Sqrt(2 * Math.Pow(Math.Max(size.Width, size.Height), 2));
                DrawNodes(g, average_node);
                DrawArrow(g, average_node);
            }
        }

        void DrawNodes(Graphics g,float average_node)
        {
            float radius = average_node / 2;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            int counter = 0;
            if (nodes.Count != 0)
            {
                for (int i = 0; i < position_nodes.Count; i += 2, counter++)
                {
                    Drawing1.DrawItemOfSet(g, nodes[counter], average_node, position_nodes[i] * width_square - radius, position_nodes[i + 1] * height_square - radius);
                }
            }
            else
            {
                g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                for (int i = 0; i < position_nodes.Count; i += 2, counter++)
                {
                    Drawing1.DrawItemOfSet(g, nodes1[counter], average_node, position_nodes[i] * width_square - radius, position_nodes[i + 1] * height_square - radius, this.Font);
                }
            }
        }

        void DrawArrow(Graphics g,float average_node)
        {
            List<PointF> points_of_control_polygon;
            float radius_circle = average_node / 2;
            float length_arrow = radius_circle * 0.8f;
            bool with_line = true;
            for (int i = 0; i < arrow_between_items.Count; i += 3)
            {
                points_of_control_polygon = GetPointsOfControlPolygon((int)arrow_between_items[i], (int)arrow_between_items[i + 1], position_nodes, radius_circle);

                if (points_of_control_polygon.Count == 2) Drawing1.DrawArrow(g, points_of_control_polygon[0], points_of_control_polygon[1], length_arrow, 1, (Brush)arrow_between_items[i + 2], true, with_line);
                else Drawing1.DrawArrow(g, points_of_control_polygon[0], points_of_control_polygon[1], points_of_control_polygon[2], points_of_control_polygon[3], length_arrow, 1, (Brush)arrow_between_items[i + 2], true, with_line);
            }
        }
        
        //return two or four points
        private List<PointF> GetPointsOfControlPolygon(int node1, int node2,List<float> position_items,float radius_circle)
        {
            PointF p1;
            PointF p2;
            PointF p11;
            PointF p22;
            List<PointF> points_of_control_polygon = new List<PointF>();

            p1 = new PointF(position_items[(node1 * 2)] * width_square, position_items[(node1 * 2) + 1] * height_square);
            p2 = new PointF(position_items[(node2* 2)] * width_square, position_items[(node2 * 2) + 1] * height_square);
            p11 = p1;
            p22 = p2;

            if (node1 == node2)
            {
                float move_x = 3.5f * radius_circle;
                float move_y = radius_circle;

                if (node1 % 2 == 0) move_x = - move_x;

                PointF p3 = new PointF(p1.X + move_x, p1.Y - move_y);
                PointF p4 = new PointF(p1.X + move_x, p1.Y + move_y);
                points_of_control_polygon.Add(Drawing1.IntersectionBC(p3, p1, p1.X, p1.Y, radius_circle));
                points_of_control_polygon.Add(p3);
                points_of_control_polygon.Add(p4);
                points_of_control_polygon.Add(Drawing1.IntersectionBC(p4, p1, p1.X, p1.Y, radius_circle));
            }
            else
            {
                float v_node1 = position_nodes[node1 * 2];float h_node1 = position_nodes[node1 * 2 + 1];
                float v_node2 = position_nodes[node2 * 2];float h_node2 = position_nodes[node2 * 2 + 1];
                if (v_node1 == v_node2)
                {
                    if (h_node1 < h_node2) { p1.X -= radius_circle / 4;p2.X -= radius_circle/4; }
                    else { p1.X += radius_circle / 4; p2.X += radius_circle / 4; }
                }
                else if (h_node1 == h_node2)
                {
                    if (v_node1 < v_node2) { p1.Y += radius_circle / 4; p2.Y += radius_circle / 4; }
                    else { p1.Y -= radius_circle / 4; p2.Y -= radius_circle / 4; }
                }
                else
                {
                    if (v_node1 < v_node2) { p1.X += radius_circle / 2; p1.Y += radius_circle / 2; p2.X -= radius_circle / 2; p2.Y -= radius_circle / 2; }
                    else { p1.Y -= radius_circle; p2.X += radius_circle; }
                }

                points_of_control_polygon.Add(Drawing1.IntersectionBC(p2, p1, p11.X, p11.Y, radius_circle));
                points_of_control_polygon.Add(Drawing1.IntersectionBC(p1, p2, p22.X, p22.Y, radius_circle));
            }
            return points_of_control_polygon;
        }

        public bool AddNode(Brush color, int number_h_line, int number_v_line)
        {
            if (nodes1.Count != 0) return false;
            else
            {
                if (number_h_line >= 0 && number_v_line >= 0 && number_h_line < horizontal_lines && number_v_line < vertical_lines)
                {
                    nodes.Add(color);
                    count_items = nodes.Count;
                    position_nodes.Add(number_v_line + 1); position_nodes.Add(number_h_line + 1);
                    return true;
                }
                else return false;
            }
        }

        public bool AddNode(string name, int number_h_line, int number_v_line)
        {
            if (nodes.Count != 0) return false;
            else
            {
                if (number_h_line >= 0 && number_v_line >= 0 && number_h_line < horizontal_lines && number_v_line < vertical_lines)
                {
                    nodes1.Add(name);
                    count_items = nodes1.Count;
                    position_nodes.Add(number_v_line + 1); position_nodes.Add(number_h_line + 1);
                    return true;
                }
                else return false;
            }
        }

        public void AddArrow(int first_item, int second_item, Brush color_arrow)
        {
            if (count_items > first_item && count_items > second_item)
            {
                if (!IsIn(first_item, second_item))
                {
                    arrow_between_items.Add(first_item);
                    arrow_between_items.Add(second_item);
                    arrow_between_items.Add(color_arrow);

                    this.Surface.Invalidate();
                }
            }
            else throw new Exception("Hodnota jednoho z parametru je vetsi nez pocet prvku v grafu");
        }

        public bool RemoveArrow(int first_item, int second_item)
        {
            for (int i = 0; i < arrow_between_items.Count; i += 3)
            {
                if ((int)arrow_between_items[i] == first_item && (int)arrow_between_items[i + 1] == second_item)
                {
                    arrow_between_items.RemoveRange(i, 3);
                    this.Surface.Invalidate();
                    return true;
                }
            }

            return false;
        }

        private bool IsIn(int first_item, int second_item)
        {
            for (int i = 0; i < arrow_between_items.Count; i+=3)
            {
                if ((int)arrow_between_items[i] == first_item && (int)arrow_between_items[i + 1] == second_item) return true;
            }

            return false;
        }

        private void SurfaceResize(object sender, EventArgs e)
        {
            SetSizeSquare();
            Surface.Invalidate();
        }
    }
}
