﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class InverseRelation : UserControl
    {
        public InverseRelation()
        {
            InitializeComponent();
            set = new Set("X");
            R1 = new Relation("R");
            R3 = new Relation("L");

            List<string> l = new List<string>();
            l.Add("a");
            l.Add("b");
            l.Add("c");

            l.ForEach(item => { set.AddItemToSet(item); });
            this.selectorPair1.AddItems(l, l);

            this.matrix1.AddNamesItems(l, l, "R");
  
            this.matrix3.AddNamesItems(l, l, "L");
            graph1 = new Graph1();
            graph1.SetGrid(3, 3);
            graph1.AddNode("a", 2, 0);
            graph1.AddNode("b", 2, 2);
            graph1.AddNode("c", 0, 0);
            graph1.Dock = DockStyle.Fill;

            graph3 = new Graph1();
            graph3.SetGrid(3, 3);
            graph3.Dock = DockStyle.Fill;
            graph3.AddNode("a", 2, 0);
            graph3.AddNode("b", 2, 2);
            graph3.AddNode("c", 0, 0);

            free_colors = new List<Brush>();
            free_colors.Add(Brushes.Black);
            free_colors.Add(Brushes.Red);
            free_colors.Add(Brushes.Green);
            free_colors.Add(Brushes.Brown);
            free_colors.Add(Brushes.Cyan);
            free_colors.Add(Brushes.Blue);
            free_colors.Add(Brushes.Gray);
            free_colors.Add(Brushes.Orange);
            free_colors.Add(Brushes.LightGreen);

        }

        Set set;
        Relation R1;
        Relation R3;
        Graph1 graph1;
        Graph1 graph3;
        List<Brush> free_colors;

        private void Relation1Paint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, R1, this.Font);
        }

        private void Relation3Paint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, R3, this.Font);
        }

        public Font GetMaxFont()
        {
            GroupBox l = this.DescriptionGraph3;
            return Drawing1.GetFont(l.CreateGraphics(), l.Width, l.Height, "M" + l.Text + "M", this.Font);
        }

        private void AddItemClick(object sender, EventArgs e)
        {
            Point items = this.selectorPair1.GetCurrentItems;
            if (R1.GetNumberOfItemOfRelation(items.X, items.Y) == -1)
            {
                Brush color = free_colors[0];
                Pen p = new Pen(color);
                R1.AddItemToRelation(items.X, items.Y, color);
                graph1.AddArrow(items.X, items.Y, color);
                this.matrix1.PutValueToItem(items.X, items.Y, "1", p.Color);
                R3.AddItemToRelation(items.Y, items.X, color);
                graph3.AddArrow(items.Y, items.X, color);
                this.matrix3.PutValueToItem(items.Y, items.X, "1", p.Color);
                free_colors.RemoveAt(0);
                Relation1.Invalidate();
                Relation3.Invalidate();
            }
            else this.selectorPair1.Signal();

            SetButtons();
        }

        private void RemoveItemClick(object sender, EventArgs e)
        {
            Point items = this.selectorPair1.GetCurrentItems;
            int number_of_item;
            if ((number_of_item = R1.GetNumberOfItemOfRelation(items.X, items.Y)) != -1)
            {
                free_colors.Add(R1.GetColorItemOfRelation(number_of_item));
                R1.RemoveItemFromRelation(items.X, items.Y);
                graph1.RemoveArrow(items.X, items.Y);
                this.matrix1.PutValueToItem(items.X, items.Y, "0", Color.Black);
                R3.RemoveItemFromRelation(items.Y, items.X);
                graph3.RemoveArrow(items.Y, items.X);
                this.matrix3.PutValueToItem(items.Y, items.X, "0", Color.Black);
                Relation1.Invalidate();
                Relation3.Invalidate();
            }
            else this.selectorPair1.Signal();

            SetButtons();
        }

        void SetButtons()
        {
            bool h;
            if (R1.CountItemsInRelation() == 0) h = false;
            else h = true;
            this.RemoveItem.Enabled = h;

            if (R1.CountItemsInRelation() >= set.CountItemsInSet() * set.CountItemsInSet()) this.AddItem.Enabled = false;
            else this.AddItem.Enabled = true;
        }

        private void radioButton1CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton1.Checked == true)
            {
                Drawing1.SetAllControlsBackColor(this.matrix1, this.BackColor);
                Drawing1.SetAllControlsBackColor(this.matrix3, this.BackColor);
                this.DescriptionGraph1.Controls.Remove(this.graph1);
                this.DescriptionGraph1.Controls.Add(this.matrix1);
                this.DescriptionGraph1.Text = "Matice reprezentující relaci R";
                this.DescriptionGraph3.Controls.Remove(this.graph3);
                this.DescriptionGraph3.Controls.Add(this.matrix3);
                this.DescriptionGraph3.Text = "Matice reprezentující relaci L";
            }
        }

        private void radioButton2CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton2.Checked == true)
            {
                Drawing1.SetAllControlsBackColor(this.graph1, this.BackColor);
                Drawing1.SetAllControlsBackColor(this.graph3, this.BackColor);
                this.DescriptionGraph1.Controls.Remove(this.matrix1);
                this.DescriptionGraph1.Controls.Add(this.graph1);
                this.DescriptionGraph1.Text = "Graf reprezentující relaci R";
                this.DescriptionGraph3.Controls.Remove(this.matrix3);
                this.DescriptionGraph3.Controls.Add(this.graph3);
                this.DescriptionGraph3.Text = "Graf reprezentující relaci L";
            }
        }

        private void InverseRelation_Resize(object sender, EventArgs e)
        {
            this.Relation1.Invalidate();
            this.Relation3.Invalidate();
        }
    }
}
