﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class Matrix : UserControl
    {
        public Matrix()
        {
            InitializeComponent();

            first_round_bracket = new PictureBox();
            first_round_bracket.BackColor = this.RowsAndColumns.BackColor;
            first_round_bracket.Dock = DockStyle.Fill;
            first_round_bracket.Visible = true;
            first_round_bracket.Paint += new PaintEventHandler(first_round_bracket_paint);

            second_round_bracket = new PictureBox();
            second_round_bracket.BackColor = this.RowsAndColumns.BackColor;
            second_round_bracket.Dock = DockStyle.Fill;
            second_round_bracket.Visible = true;
            second_round_bracket.Paint += new PaintEventHandler(second_round_bracket_paint);

        }

        PictureBox first_round_bracket;
        PictureBox second_round_bracket;
        int count_items_on_row;

        public void SetRows(int count)
        {
            this.RowsAndColumns.RowCount = count + 4;
            float width_row = 100 / this.RowsAndColumns.RowCount;
            this.RowsAndColumns.RowStyles[0].SizeType = SizeType.Percent;
            this.RowsAndColumns.RowStyles[0].Height = width_row;

            for (int i = 1; i < this.RowsAndColumns.RowCount; i++)
            {
                this.RowsAndColumns.RowStyles.Add(new RowStyle(SizeType.Percent, width_row));
            }
        }

        public void SetColumns(int count)
        {
            this.RowsAndColumns.ColumnCount = count + 7;
            float width_column = 100 / this.RowsAndColumns.ColumnCount;
            this.RowsAndColumns.ColumnStyles[0].SizeType = SizeType.Percent;
            this.RowsAndColumns.ColumnStyles[0].Width = width_column;

            for (int i = 1; i < this.RowsAndColumns.ColumnCount; i++)
            {
                this.RowsAndColumns.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, width_column));
            }

        }

        public void AddNamesItems(List<string> rows, List<string> columns,string name_matrix)
        {
            count_items_on_row = columns.Count;
            SetRows(rows.Count);
            SetColumns(count_items_on_row);
            TableLayoutControlCollection cells = this.RowsAndColumns.Controls;
            Color c = this.RowsAndColumns.BackColor;
            int rows_count = this.RowsAndColumns.RowCount;
            int columns_count = this.RowsAndColumns.ColumnCount;
            Label cell;
            int counter = 3;

            for (int i = 3; i < rows_count - 1; i++)
            {
                for (int j = 5; j < columns_count - 2; j++)
                {
                    cell = new Label();
                    cell.Dock = DockStyle.Fill;
                    cell.Visible = true;
                    cell.BackColor = c;
                    cell.Text = "0";
                    cell.TextAlign = ContentAlignment.MiddleCenter;
                    cell.AutoSize = true;
                    cells.Add(cell, j, i);
                }
            }

            rows.ForEach(i =>
            {
                cell = new Label();
                cell.Dock = DockStyle.Fill;
                cell.Visible = true;
                cell.BackColor = Color.Beige;
                cell.BorderStyle = BorderStyle.Fixed3D;
                Padding p = cell.Margin;
                p.Bottom = 1;
                p.Top = 1;
                p.Left = 1;
                p.Right = 1;
                cell.Margin = p;
                cell.Text = i;
                cell.TextAlign = ContentAlignment.MiddleCenter;
                cell.AutoSize = true;
                cells.Add(cell, 1, counter);
                counter++;
            });

            counter = 5;

            columns.ForEach(i =>
            {
                cell = new Label();
                cell.Dock = DockStyle.Fill;
                cell.Visible = true;
                cell.BackColor = Color.Beige;
                cell.BorderStyle = BorderStyle.Fixed3D;
                Padding p = cell.Margin;
                p.All = 0;
                p.Bottom = 1;
                p.Top = 1;
                p.Left = 0;
                p.Right = 0;
                cell.Margin = p;
                cell.Text = i;
                cell.TextAlign = ContentAlignment.MiddleCenter;
                cell.AutoSize = true;
                cells.Add(cell, counter, 1);
                counter++;
            });

            int back = 0;
            int rowspan = 1;
            if (rows.Count % 2 == 0) { back = 1; rowspan = 2; }

            cell = new Label();
            cell.Dock = DockStyle.Fill;
            cell.Visible = true;
            cell.BackColor = c;
            cell.Text = name_matrix;
            cell.TextAlign = ContentAlignment.MiddleLeft;
            cell.AutoSize = true;
            cells.Add(cell, 2, rows.Count / 2 + 3 - back);
            this.RowsAndColumns.SetRowSpan(cell, rowspan);

            cell = new Label();
            cell.Dock = DockStyle.Fill;
            cell.Visible = true;
            cell.BackColor = c;
            cell.Text = "=";
            cell.TextAlign = ContentAlignment.MiddleLeft;
            cell.AutoSize = true;
            cells.Add(cell, 3, rows.Count / 2 + 3 - back);
            this.RowsAndColumns.SetRowSpan(cell, rowspan);

            cells.Add(first_round_bracket, 4, 3);
            this.RowsAndColumns.SetRowSpan(first_round_bracket, rows.Count);

            cells.Add(second_round_bracket, columns_count - 2, 3);
            this.RowsAndColumns.SetRowSpan(second_round_bracket, rows.Count);
        }

        public void PutValueToItem(int row, int column, string value, Color color_item)
        {
            int label_position = row * count_items_on_row + column;
            this.RowsAndColumns.Controls[label_position].Text = value;
            this.RowsAndColumns.Controls[label_position].ForeColor = color_item;
        }

        public void SignOrUnsignItem(int row, int column, bool sign)
        {
            int label_position = row * count_items_on_row + column;
            if (sign)
            {
                ((Label)this.RowsAndColumns.Controls[label_position]).BorderStyle = BorderStyle.FixedSingle;
                //((Label)this.RowsAndColumns.Controls[label_position]).BackColor = color;
            }
            else
            {
                ((Label)this.RowsAndColumns.Controls[label_position]).BorderStyle = BorderStyle.None;
                //((Label)this.RowsAndColumns.Controls[label_position]).BackColor = Color.Transparent;
            }
        }

        public Rectangle GetSizeAndPositionOfItem(int row, int column)
        {
            Label item = (Label)this.RowsAndColumns.Controls[row * count_items_on_row + column];
            return new Rectangle(item.Location.X, item.Location.Y, item.Size.Width, item.Size.Height);
        }

        private void first_round_bracket_paint(object sender, PaintEventArgs e)
        {
            PictureBox pcb = (PictureBox)sender;
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            Pen p = new Pen(Brushes.Black);
            p.Width = 2;
            g.DrawBezier(p, pcb.Width, 0, pcb.Width / 2, (int)(pcb.Height * 0.1F), pcb.Width / 2, (int)(pcb.Height - pcb.Height * 0.1F), pcb.Width, pcb.Height);
        }

        private void second_round_bracket_paint(object sender, PaintEventArgs e)
        {
            PictureBox pcb = (PictureBox)sender;
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            Pen p = new Pen(Brushes.Black);
            p.Width = 2;
            g.DrawBezier(p, 0, 0, pcb.Width / 2, (int)(pcb.Height * 0.1F), pcb.Width / 2, (int)(pcb.Height - pcb.Height * 0.1F), 0, pcb.Height);
        }

        public override Font Font
        {
            get
            {
                return base.Font;
            }
            set
            {
                //base.Font = value;
                if(this.RowsAndColumns.Controls.Count != 0) ChangeFontItems(value);
            }
        }

        private void ChangeFontItems(Font font)
        {
            TableLayoutControlCollection cc = this.RowsAndColumns.Controls;
            for (int i = 0; i < cc.Count - 2; i++) cc[i].Font = font;
        }

        public Font GetMaxFont()
        {
            Label l = (Label)this.RowsAndColumns.Controls[0];
            Graphics g = l.CreateGraphics();
            return Drawing1.GetFont(g, l.Width, l.Height, l.Text, l.Font);
        }

        private void RowsAndColumns_Resize(object sender, EventArgs e)
        {
            first_round_bracket.Invalidate();
            second_round_bracket.Invalidate();
        }
    }
}
