﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class OrderRelation2 : UserControl
    {
        public OrderRelation2()
        {
            InitializeComponent();

            List<string> items = new List<string>(){"a","b","c","d","e","f","g","h","i","j","k"};
            set = new Set("X");
            items.ForEach(item=> set.AddItemToSet(item));
            r_u = new Relation("R"); r_p = new Relation("L");
            c_r = new CreateRelation();
            c_r.EquivalenceRelation = false;
            c_s = new CreateSet();
            c_s.SetMaxCountItems = 15;
        }

        Set set;
        Relation r_u;
        Relation r_p;
        CreateRelation c_r;
        CreateSet c_s;
        bool is_order_relation;

        private void DrawRPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            if (is_order_relation == true) Drawing1.DrawRelation(p, g, rect, set, set, r_u, this.Font);
            else
            {
                p.AutoScrollMinSize = new Size((int)g.MeasureString("Vámi vytvořená relace není relace uspořádání", this.Font).Width + v_space, 0);
                g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                g.DrawString("Vámi vytvořená relace není relace uspořádání", this.Font, Brushes.Black, v_space + p.AutoScrollPosition.X, (int)(p.Height * 0.1f));
            }
        }

        private void DrawLPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            if (is_order_relation == true) Drawing1.DrawRelation(p, g, rect, set, set, r_p, this.Font);
        }

        private void CreateRClick(object sender, EventArgs e)
        {
            c_r.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_r, this.BackColor);
            c_r.Relation1 = r_u;
            c_r.Set = set;
            if (c_r.ShowDialog() == DialogResult.OK)
            {
                r_u = c_r.Relation1;
                if (Computing.IsReflexive(r_u, set) && Computing.IsAntiSymmetry(r_u) && Computing.IsTransitive(r_u))
                {
                    is_order_relation = true;
                    r_p = Computing.CoverageRelation(r_u, "L");
                    this.diagram1.SetAsHasseDiagram(set, r_u, set.CountItemsInSet());
                }
                else
                {
                    this.diagram1.Clear();
                    is_order_relation = false;
                }
                this.DrawR.Invalidate();
                this.DrawL.Invalidate();
                this.diagram1.DiagramPaint();
            }
        }

        private void DescriptionSetPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width - v_space, p.Height), set, this.Font);
        }

        private void DescriptionSetResize(object sender, EventArgs e)
        {
            this.DrawR.Invalidate();
            this.DrawL.Invalidate();
            this.DescriptionSet.Invalidate();
        }

        private void CreateSetClick(object sender, EventArgs e)
        {
            c_s.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_s, this.BackColor);
            c_s.Set = set;
            DialogResult d = c_s.ShowDialog();
            if (d == DialogResult.OK)
            {
                set = c_s.Set;
                r_u.RemoveAllItems();
                r_p.RemoveAllItems();
                if (set.CountItemsInSet() == 0) is_order_relation = true;
                else is_order_relation = false;
                this.diagram1.Clear();
                this.DrawR.Invalidate();
                this.DrawL.Invalidate();
                this.DescriptionSet.Invalidate();
                this.diagram1.DiagramPaint();
            }
        }
    }
}
