﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class OrderRelation3 : UserControl
    {
        public OrderRelation3()
        {
            InitializeComponent();
            items = new List<string>() { "a", "b", "c", "d", "e", "f", "g", "h", "i" };
            set = new Set("X");
            items.ForEach(item => set.AddItemToSet(item));
            r = Computing.CreateRandomOrderRelation(set, "R");
            this.diagram1.SetAsHasseDiagram(set, r, set.CountItemsInSet());
            this.selectorItem1.AddItem(items);
            items_c = new List<int>();
            sup = inf = -1;
            is_order_relation = true;
            c_r = new CreateRelation();
            c_r.EquivalenceRelation = false;
            c_s = new CreateSet();
            c_s.SetMaxCountItems = 15;
        }

        Set set;
        Relation r;
        List<int> items_c;
        List<int> items_h;
        List<int> items_d;
        List<string> items;
        bool is_order_relation;
        int sup, inf;
        CreateRelation c_r;
        CreateSet c_s;

        private void DrawORPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            if (is_order_relation == true) Drawing1.DrawRelation(p, g, rect, set, set, r, p.Font);
            else
            {
                p.AutoScrollMinSize = new Size((int)g.MeasureString("Vámi vytvořená relace není relace uspořádání", this.Font).Width + v_space, 0);
                g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                g.DrawString("Vámi vytvořená relace není relace uspořádání", this.Font, Brushes.Black, v_space + p.AutoScrollPosition.X, (int)(p.Height * 0.1f));
            }
        }

        private void AddItemClick(object sender, EventArgs e)
        {
            int item = this.selectorItem1.GetCurrentItem;
            if (item != -1 && !items_c.Exists(item1 => item == item1))
            {
                items_c.Add(item);
                SetItems();
            }
            else this.selectorItem1.Signal();

            SetButtons();
        }

        void SetItems()
        {
            items_h = Computing.UpperCone(set.CountItemsInSet(), r, items_c);
            items_d = Computing.LowerCone(set.CountItemsInSet(), r, items_c);
            sup = items_h.FindIndex(item => items_h.Intersect(r.GetItemsInRelationWith(item, 0)).Count() >= items_h.Count);
            inf = items_d.FindIndex(item => items_d.Intersect(r.GetItemsInRelationWith(item, 1)).Count() >= items_d.Count);
            sup = sup == -1 ? sup : items_h[sup];
            inf = inf == -1 ? inf : items_d[inf];
            SetLabels();
            SetDiagram();
        }

        void SetLabels()
        {
            int count_items;
            count_items = items_c.Count;
            this.DescriptionC.Text = "Množina C = { ";
            if (count_items == 0) this.DescriptionC.Text += " }";
            else
            {
                for (int i = 0; i < count_items - 1; i++) this.DescriptionC.Text += set.GetNameItem(items_c[i]) + ", ";
                this.DescriptionC.Text += set.GetNameItem(items_c[count_items - 1]) + " }";
            }
            count_items = items_h.Count;
            this.DescriptionH.Text = "Horní kužel C = { ";
            if (count_items == 0) this.DescriptionH.Text += " }";
            else
            {
                for (int i = 0; i < count_items - 1; i++) this.DescriptionH.Text += set.GetNameItem(items_h[i]) + ", ";
                this.DescriptionH.Text += set.GetNameItem(items_h[count_items - 1]) + " }";
            }
            count_items = items_d.Count;
            this.DescriptionD.Text = "Dolní kužel C = { ";
            if (count_items == 0) this.DescriptionD.Text += " }";
            else
            {
                for (int i = 0; i < count_items - 1; i++) this.DescriptionD.Text += set.GetNameItem(items_d[i]) + ", ";
                this.DescriptionD.Text += set.GetNameItem(items_d[count_items - 1]) + " }";
            }

            if (sup == -1) this.DescriptionS.Text = "Supremum množiny C:";
            else this.DescriptionS.Text = "Supremum množiny C: " + set.GetNameItem(sup);

            if (inf == -1) this.DescriptionI.Text = "Infimum množiny C:";
            else this.DescriptionI.Text = "Infimum množiny C: " + set.GetNameItem(inf);
        }

        void SetDiagram()
        {
            this.diagram1.SignsClear();
            if (sup != -1 && sup == inf)
            {
                this.diagram1.SetSign(set.GetNameItem(sup), Pens.Red, Diagram.SizeSign.Small);
                this.diagram1.SetSign(set.GetNameItem(sup), Pens.Green, Diagram.SizeSign.Medium);
                this.diagram1.SetSign(set.GetNameItem(sup), Pens.Blue, Diagram.SizeSign.Large);
            }
            else
            {
                if (items_c.Exists(item => item == sup))
                {
                    this.diagram1.SetSign(set.GetNameItem(sup), Pens.Red, Diagram.SizeSign.Small);
                    this.diagram1.SetSign(set.GetNameItem(sup), Pens.Green, Diagram.SizeSign.Medium);
                }
                else sup = -1;

                if (items_c.Exists(item => item == inf))
                {
                    this.diagram1.SetSign(set.GetNameItem(inf), Pens.Red, Diagram.SizeSign.Small);
                    this.diagram1.SetSign(set.GetNameItem(inf), Pens.Blue, Diagram.SizeSign.Medium);
                }
                else inf = -1;
            }
            items_c.ForEach(item => { if (item != sup && item != inf)this.diagram1.SetSign(set.GetNameItem(item), Pens.Red, Diagram.SizeSign.Small); });
            items_h.ForEach(item => { if (item != sup)this.diagram1.SetSign(set.GetNameItem(item), Pens.Green, Diagram.SizeSign.Small); });
            items_d.ForEach(item => { if (item != inf)this.diagram1.SetSign(set.GetNameItem(item), Pens.Blue, Diagram.SizeSign.Small); });
            this.diagram1.DiagramPaint();
        }

        private void RemoveItemClick(object sender, EventArgs e)
        {
            int item = this.selectorItem1.GetCurrentItem;
            if (items_c.Exists(item1 => item == item1))
            {
                items_c.Remove(item);
                SetItems();
            }
            else this.selectorItem1.Signal();

            SetButtons();
        }

        private void RemoveAllItemsClick(object sender, EventArgs e)
        {
            items_c.Clear();
            SetItems();

            SetButtons();
        }

        void SetButtons()
        {
            if (items_c.Count == 0)
            {
                this.RemoveItem.Enabled = this.RemoveAllItems.Enabled = false;
            }
            else
            {
                this.RemoveItem.Enabled = this.RemoveAllItems.Enabled = true;
            }

            if (items_c.Count == set.CountItemsInSet()) this.AddItem.Enabled = false;
            else this.AddItem.Enabled = true;
        }

        private void CreateRelationClick(object sender, EventArgs e)
        {
            c_r.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_r, this.BackColor);
            c_r.Relation1 = r;
            c_r.Set = set;
            bool help_item;
            if (c_r.ShowDialog() == DialogResult.OK)
            {
                r = c_r.Relation1;
                if (Computing.IsReflexive(r, set) && Computing.IsAntiSymmetry(r) && Computing.IsTransitive(r))
                {
                    help_item = true;
                    this.diagram1.SetAsHasseDiagram(set, r, set.CountItemsInSet());
                    if (set.CountItemsInSet() > 0) this.ArrangementSetC.Enabled = true;
                    else this.ArrangementSetC.Enabled = false;
                }
                else
                {
                    this.diagram1.Clear();
                    r.RemoveAllItems();
                    help_item = false;
                    this.ArrangementSetC.Enabled = false;
                }
                is_order_relation = help_item;
                this.DescriptionHasseDiagram.Enabled = this.DescriptionC.Enabled = this.DescriptionD.Enabled = this.DescriptionH.Enabled = help_item;
                this.DescriptionS.Enabled = this.DescriptionI.Enabled = help_item;
                items_c.Clear();
                SetButtons();
                SetItems();
                this.DrawOR.Invalidate();
                this.diagram1.DiagramPaint();
            }
        }

        private void DrawSetPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width - v_space, p.Height), set, this.Font);
        }

        private void OrderRelation3Resize(object sender, EventArgs e)
        {
            this.DrawOR.Invalidate();
            this.DrawSet.Invalidate();
        }

        private void CreateSetClick(object sender, EventArgs e)
        {
            c_s.Font = this.Font;
            Drawing1.SetAllControlsBackColor(c_s, this.BackColor);
            c_s.Set = set;
            DialogResult d = c_s.ShowDialog();
            bool help_item;
            if (d == DialogResult.OK)
            {
                set = c_s.Set;
                this.diagram1.Clear();
                r.RemoveAllItems();
                items.Clear();
                for (int i = 0; i < set.CountItemsInSet(); i++) items.Add(set.GetNameItem(i));
                this.selectorItem1.AddItem(items);
                if (set.CountItemsInSet() == 0) help_item = true;
                else help_item = false;

                this.ArrangementSetC.Enabled = false; 
                items_c.Clear();
                SetButtons();
                SetItems();
                this.DrawOR.Invalidate();
                this.diagram1.DiagramPaint();
                this.DrawSet.Invalidate();
                is_order_relation = help_item;
                this.DescriptionHasseDiagram.Enabled = this.DescriptionC.Enabled = this.DescriptionD.Enabled = this.DescriptionH.Enabled = help_item;
                this.DescriptionS.Enabled = this.DescriptionI.Enabled = help_item;
            }
        }
    }
}
