﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class RepreRelationSetGraphMatrix : UserControl
    {
        public RepreRelationSetGraphMatrix()
        {
            InitializeComponent();

            List<string> items = new List<string>();
            items.Add("a");
            items.Add("b");
            items.Add("c");
            items.Add("d");

            set = new Set("X");
            relation = new Relation("R");

            items.ForEach(item => set.AddItemToSet(item));
            this.graph1.SetGrid(3, 3);
            this.graph1.AddNode("a", 2, 0);
            this.graph1.AddNode("b", 2, 2);
            this.graph1.AddNode("c", 0, 2);
            this.graph1.AddNode("d", 0, 0);
            this.matrix1.AddNamesItems(items, items, "K");
            this.selectorPair1.AddItems(items, items);
            SetButtons();
        }

        Relation relation;
        Set set;

        public Font GetMaxFont()
        {
            GroupBox l = this.CaptionRelation;
            Graphics g = l.CreateGraphics();
            return Drawing1.GetFont(g, l.Width, l.Height, "\t" + l.Text + "\t", this.Font);
        }

        private void AddItemClick(object sender, EventArgs e)
        {
            Point pair = this.selectorPair1.GetCurrentItems;
            if (relation.AddItemToRelation(pair.X, pair.Y, Brushes.Black) == false) this.selectorPair1.Signal();
            else
            {
                this.graph1.AddArrow(pair.X, pair.Y, Brushes.Black);
                this.matrix1.PutValueToItem(pair.X, pair.Y, "1", Color.Black);
                this.Relation.Invalidate();
            }
            SetButtons();
        }

        private void RemoveItemClick(object sender, EventArgs e)
        {
            Point pair = this.selectorPair1.GetCurrentItems;
            if (relation.RemoveItemFromRelation(pair.X, pair.Y) == false) this.selectorPair1.Signal();
            else
            {
                this.graph1.RemoveArrow(pair.X, pair.Y);
                this.matrix1.PutValueToItem(pair.X, pair.Y, "0", Color.Black);
                this.Relation.Invalidate();
            }
            SetButtons();
        }

        void SetButtons()
        {
            bool h;
            if (relation.CountItemsInRelation() == 0) h = false;
            else h = true;
            this.RemoveItem.Enabled = h;

            if (relation.CountItemsInRelation() >= set.CountItemsInSet() * set.CountItemsInSet()) this.AddItem.Enabled = false;
            else this.AddItem.Enabled = true;
        }

        private void RelationPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.DrawRelation(p, g, rect, set, set, relation, p.Font);
        }

        private void RelationResize(object sender, EventArgs e)
        {
            this.Relation.Invalidate();
        }
    }
}
