﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public delegate void DelegateChanged(object sender);
    public enum DrawStringOrColor { String,Color};

    partial class SelectorItem : UserControl
    {
        public enum PositionButtons1{left,right};

        public SelectorItem()
        {
            InitializeComponent();
            t_t = new ToolTip();
            position_buttons = PositionButtons1.left;
            current_item = -1;
            items1 = new List<Brush>();
            items2 = new List<string>();
            draw_string_or_color = DrawStringOrColor.String;
            pictureBox1.Paint += new PaintEventHandler(this.pictureBox1Paint2);
            SetActionsForButtons();
        }

        PositionButtons1 position_buttons;
        List<Brush> items1;
        List<string> items2;
        int current_item;
        int count_items;
        DrawStringOrColor draw_string_or_color;
        ToolTip t_t;
        public event DelegateChanged ChangedItem;

        public void AddItem(List<Brush> items)
        {
            if (draw_string_or_color == DrawStringOrColor.Color)
            {
                items1.Clear();
                items.ForEach(item => items1.Add((Brush)item.Clone()));
                current_item = 0;
                count_items = items1.Count;
                if (count_items == 0) current_item = -1;
                this.pictureBox1.Invalidate();
            }
            else throw new Exception("Prvky musi byt typu Brush, pokud chces vkladat prvky typu string, nastav vlastnost DrawStringOrColor na String");
        }

        public void AddItem(List<string> items)
        {
            if (draw_string_or_color == DrawStringOrColor.String)
            {
                items2.Clear();
                items.ForEach(item => items2.Add(item));
                current_item = 0;
                count_items = items2.Count;
                if (count_items == 0) current_item = -1;
                this.pictureBox1.Invalidate();
            }
            else throw new Exception("Prvky musi byt typu String, pokud chces vkladat prvky typu Brush, nastav vlastnost DrawStringOrColor na Brush");
        }

        [Category("Appearance")]
        [Description("Specifies type of items")]
        public DrawStringOrColor DrawStringOrColor
        {
            get{return draw_string_or_color;}
            set
            {
                if (value != draw_string_or_color)
                {
                    if (value == DrawStringOrColor.String)
                    {
                        pictureBox1.Paint -= new PaintEventHandler(this.pictureBox1Paint1);
                        pictureBox1.Paint += new PaintEventHandler(this.pictureBox1Paint2);
                        draw_string_or_color = value;
                    }
                    else
                    {
                        pictureBox1.Paint -= new PaintEventHandler(this.pictureBox1Paint2);
                        pictureBox1.Paint += new PaintEventHandler(this.pictureBox1Paint1);
                        draw_string_or_color = value;
                    }
                }
            }
        }

        public void SetColorItems()
        {
            pictureBox1.Paint += new PaintEventHandler(this.pictureBox1Paint1);
            SetActionsForButtons();
        }

        public int GetCurrentItem
        {
            get { return current_item; }
        }

        public override Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            set
            {
                base.BackColor = value;
                this.tableLayoutPanel1.BackColor = value;
                this.pictureBox1.BackColor = value;
            }
        }

        /// <summary>
        /// Set width buttons in per cents from whole width of control
        /// </summary>
        /// <param name="width"></param>
        public void SetWidthButtonsInPercents(float width)
        {
            //if (width > 0 && width < 100)
            //{
                if (position_buttons == PositionButtons1.left)
                {
                    tableLayoutPanel1.ColumnStyles[0].Width = width;
                    tableLayoutPanel1.ColumnStyles[1].Width = 100 - width;
                }
                else
                {
                    tableLayoutPanel1.ColumnStyles[0].Width = 100 - width;
                    tableLayoutPanel1.ColumnStyles[1].Width = width;
                }
           // }
           // else throw new Exception("Hodnota width je v procentech a musi byt v rozmezi 1 az 99");
        }

        [Category("Appearance")]
        [Description("Specifies left or right position buttons")]

        public PositionButtons1 PositionButtons
        {
            get { return position_buttons; }
            set
            {
                if (value != position_buttons)
                {
                    if (value == PositionButtons1.left)
                    {
                        position_buttons = PositionButtons1.left;
                        tableLayoutPanel1.ColumnStyles[0].Width = 30F;
                        tableLayoutPanel1.ColumnStyles[1].Width = 70F;
                        PictureBox p = pictureBox1;
                        Button b1 = aiButton1;
                        Button b2 = aiButton2;
                        tableLayoutPanel1.Controls.Clear();
                        tableLayoutPanel1.Controls.Add(p, 1, 0);
                        tableLayoutPanel1.SetRowSpan(p, 2);
                        tableLayoutPanel1.Controls.Add(b1, 0, 0);
                        tableLayoutPanel1.Controls.Add(b2, 0, 1);
                    }
                    else
                    {
                        position_buttons = PositionButtons1.right;
                        tableLayoutPanel1.ColumnStyles[0].Width = 70F;
                        tableLayoutPanel1.ColumnStyles[1].Width = 30F;
                        PictureBox p = pictureBox1;
                        Button b1 = aiButton1;
                        Button b2 = aiButton2;
                        tableLayoutPanel1.Controls.Clear();
                        tableLayoutPanel1.Controls.Add(p, 0, 0);
                        tableLayoutPanel1.SetRowSpan(p, 2);
                        tableLayoutPanel1.Controls.Add(b1, 1, 0);
                        tableLayoutPanel1.Controls.Add(b2, 1, 1);
                    }
                }
            }
        }

        private void pictureBox1Paint1(object sender, PaintEventArgs e)
        {
            if (items1.Count != 0)
            {
                PictureBox p = (PictureBox)sender;
                Graphics g = e.Graphics;
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                float average_of_item = Drawing1.GetAverageOfItem(this.Font);
                Drawing1.DrawItemOfSet(g, items1[current_item], average_of_item, p.Width / 2 - average_of_item / 2, p.Height / 2 - average_of_item / 2);
            }
        }

        private void pictureBox1Paint2(object sender, PaintEventArgs e)
        {
            if (items2.Count != 0)
            {
                PictureBox p = (PictureBox)sender;
                Graphics g = e.Graphics;
                g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                Rectangle rect = new Rectangle(0, 0, p.Width, p.Height);
                string item = items2[current_item];
                StringFormat s = new StringFormat(); s.Alignment = s.LineAlignment = StringAlignment.Center;
                if (g.MeasureString(item, this.Font).Width > rect.Width)
                {
                    t_t.SetToolTip(this.pictureBox1, item);
                    item += "...";
                    while (g.MeasureString(item, this.Font).Width > rect.Width && item != "...") item = item.Remove(item.Length - 4, 1);
                    g.DrawString(item, this.Font, Brushes.Black, rect, s);
                }
                else
                {
                    t_t.SetToolTip(this.pictureBox1, null);
                    g.DrawString(item, this.Font, Brushes.Black, rect, s);
                }
            }
        }

        private void pictureBox1Resize(object sender, EventArgs e)
        {
            pictureBox1.Invalidate();
        }

        private void SetActionsForButtons()
        {
            this.aiButton1.Click += new EventHandler(Button1Click);
            this.aiButton2.Click += new EventHandler(Button2Click);
        }

        private void Button1Click(object sender, EventArgs e)
        {
            if (count_items != 0)
            {
                if (current_item == count_items - 1) current_item = 0;
                else current_item++;
                pictureBox1.Invalidate();
            }
            if(ChangedItem != null) ChangedItem(this);
        }

        private void Button2Click(object sender, EventArgs e)
        {
            if (count_items != 0)
            {
                if (current_item == 0) current_item = count_items - 1;
                else current_item--;
                pictureBox1.Invalidate();
            }
            if (ChangedItem != null) ChangedItem(this);
        }

        public void Signal()
        {
            timer1.Interval = 100;
            timer1.Start();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            Timer t = (Timer)sender;
            if (this.pictureBox1.BackColor == Color.Red)
            {
                this.pictureBox1.BackColor = this.BackColor;
                this.pictureBox1.ForeColor = this.ForeColor;
                t.Stop();
            }
            else
            {
                this.pictureBox1.BackColor = Color.Red;
                this.pictureBox1.ForeColor = Color.White;
            }
        }
    }
}
