﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    partial class SelectorPair : UserControl
    {
        public SelectorPair()
        {
            InitializeComponent();
            pen = new Pen(Brushes.Black);
            pen.Width = 2;
        }

        Pen pen;

        public void AddItems(List<string> items1 , List<string> items2)
        {
                this.selectorItem1.AddItem(items1);
                this.selectorItem2.AddItem(items2);
        }

        public void AddItems(List<Brush> items1, List<Brush> items2)
        {
            this.selectorItem1.AddItem(items1);
            this.selectorItem2.AddItem(items2);
        }

        public Point GetCurrentItems
        {
            get { return new Point(selectorItem1.GetCurrentItem, selectorItem2.GetCurrentItem); }
        }

        private void LeftBracketPaint(object sender, PaintEventArgs e)
        {
            PictureBox p = (PictureBox)sender;
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            //Font f = Drawing1.GetFont(g, p.Width, p.Height, "{", this.Font);
            SizeF size = g.MeasureString("{", this.Font);
            float x = p.Width / 2 - size.Width / 2;
            float y = p.Height / 2 - size.Height / 2;
            Drawing1.DrawLeftBracket(g, x, y, pen, this.Font);
        }

        private void RightBracketPaint(object sender, PaintEventArgs e)
        {
            PictureBox p = (PictureBox)sender;
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            //Font f = Drawing1.GetFont(g, p.Width, p.Height, "}", this.Font);
            SizeF size = g.MeasureString("}", this.Font);
            float x = p.Width / 2 - size.Width / 2;
            float y = p.Height / 2 - size.Height / 2;
            Drawing1.DrawRightBracket(g, x, y, pen, this.Font);
        }

        private void StrokePaint(object sender, PaintEventArgs e)
        {
            PictureBox p = (PictureBox)sender;
            Graphics g = e.Graphics;
            g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            Rectangle rect = new Rectangle(0, 0, p.Width, p.Height);
            StringFormat s = new StringFormat();
            s.Alignment = StringAlignment.Center;
            s.LineAlignment = StringAlignment.Center;
            g.DrawString(",", this.Font, Brushes.Black, rect, s);
        }

        public Font GetMaxFont
        {
            get 
            {
                Graphics g = LeftBracket.CreateGraphics();
                return Drawing1.GetFont(g, LeftBracket.Width, LeftBracket.Height, "{", this.Font);
            }
        }

        private void SelectorPairResize(object sender, EventArgs e)
        {
            LeftBracket.Invalidate();
            RightBracket.Invalidate();
            Stroke.Invalidate();
        }

        [Category("Appearance")]
        [Description("Specifies type of items in pair")]
        public DrawStringOrColor DrawStringOrColor
        {
            get { return this.selectorItem1.DrawStringOrColor; }
            set
            {
                this.selectorItem1.DrawStringOrColor = this.selectorItem2.DrawStringOrColor = value;
            }
        }

        public void Signal()
        {
            this.selectorItem1.Signal();
            this.selectorItem2.Signal();
        }
    }
}
