﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class SymmetryRelation : UserControl
    {
        public SymmetryRelation()
        {
            InitializeComponent();

            List<string> items = new List<string>() { "a", "b", "c", "d" };
            set = new Set("X");
            relation = new Relation("R");

            items.ForEach(item => set.AddItemToSet(item));

            relation.AddItemToRelation(0, 1, Brushes.Red);
            relation.AddItemToRelation(2, 1, Brushes.Black);
            relation.AddItemToRelation(1, 3, Brushes.Green);
            relation.AddItemToRelation(2, 0, Brushes.Blue);
            relation.AddItemToRelation(2, 2, Brushes.Brown);
            relation.AddItemToRelation(0, 2, Brushes.Blue);
            relation.AddItemToRelation(1, 0, Brushes.Red);
            relation.AddItemToRelation(3, 1, Brushes.Green);
            relation.AddItemToRelation(1, 2, Brushes.Black);

            graph1.SetGrid(3, 3);
            graph1.AddNode("a", 2, 0);
            graph1.AddNode("b", 2, 2);
            graph1.AddNode("c", 0, 2);
            graph1.AddNode("d", 0, 0);
            this.graph1.AddArrow(0, 1, Brushes.Red);
            this.graph1.AddArrow(2, 1, Brushes.Black);
            this.graph1.AddArrow(1, 3, Brushes.Green);
            this.graph1.AddArrow(2, 0, Brushes.Blue);
            this.graph1.AddArrow(2, 2, Brushes.Brown);
            this.graph1.AddArrow(1, 2, Brushes.Black);
            this.graph1.AddArrow(1, 0, Brushes.Red);
            this.graph1.AddArrow(3, 1, Brushes.Green);
            this.graph1.AddArrow(0, 2, Brushes.Blue);

            this.matrix1.AddNamesItems(items, items, "R");

            for (int i = 0; i < items.Count; i++)
            {
                for (int j = 0; j < items.Count; j++) this.matrix1.PutValueToItem(i, j, "0", Color.Gray);
            }

            this.matrix1.PutValueToItem(0, 1, "1", Color.Red);
            this.matrix1.PutValueToItem(2, 1, "1", Color.Black);
            this.matrix1.PutValueToItem(1, 3, "1", Color.Green);
            this.matrix1.PutValueToItem(2, 0, "1", Color.Blue);
            this.matrix1.PutValueToItem(2, 2, "1", Color.Brown);
            this.matrix1.PutValueToItem(1, 2, "1", Color.Black);
            this.matrix1.PutValueToItem(1, 0, "1", Color.Red);
            this.matrix1.PutValueToItem(3, 1, "1", Color.Green);
            this.matrix1.PutValueToItem(0, 2, "1", Color.Blue);

            actual_item = 0;

            signed_items = new List<int>() { -1, -1 };
            SetMagnifyingglass();
        }

        Relation relation;
        Set set;
        int actual_item;
        List<int> signed_items;

        private void DrawRelationPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.SignItemOfRelation(p, g, rect, relation, set, set, this.Font, signed_items, Pens.Black);
            Drawing1.DrawRelation(p, g, rect, set, set, relation, p.Font);
        }

        public Font GetMaxFont()
        {
            GroupBox l = this.CaptionRelation;
            Graphics g = l.CreateGraphics();
            return Drawing1.GetFont(g, l.Width, l.Height, "\t" + l.Text + "\t", l.Font);
        }

        private void MoveRightClick(object sender, EventArgs e)
        {
            actual_item++;
            SetMagnifyingglass();
            if (actual_item == 4) this.MoveRight.Enabled = false;
            else this.MoveLeft.Enabled = true;
        }

        private void MoveLeftClick(object sender, EventArgs e)
        {
            actual_item--;
            SetMagnifyingglass();
            if (actual_item == 0) this.MoveLeft.Enabled = false;
            else this.MoveRight.Enabled = true;
        }

        void SetMagnifyingglass()
        {
            List<int> items_of_item_of_relation =  relation.GetItemOfRelation(actual_item);
            int symmetry_item = relation.GetNumberOfItemOfRelation(items_of_item_of_relation[1], items_of_item_of_relation[0]);

                if (signed_items[0] != -1)
                {
                    List<int> items_of_item_of_relation1 = relation.GetItemOfRelation(signed_items[0]);
                    this.graph1.SetWidthArrow(items_of_item_of_relation1[0], items_of_item_of_relation1[1], 1);
                    this.graph1.SetWidthArrow(items_of_item_of_relation1[1], items_of_item_of_relation1[0], 1);
                    this.matrix1.SignOrUnsignItem(items_of_item_of_relation1[0], items_of_item_of_relation1[1], false);
                    this.matrix1.SignOrUnsignItem(items_of_item_of_relation1[1], items_of_item_of_relation1[0], false);
                }
                this.graph1.SetWidthArrow(items_of_item_of_relation[0], items_of_item_of_relation[1], 2);
                this.graph1.SetWidthArrow(items_of_item_of_relation[1], items_of_item_of_relation[0], 2);
                this.matrix1.SignOrUnsignItem(items_of_item_of_relation[0], items_of_item_of_relation[1], true);
                this.matrix1.SignOrUnsignItem(items_of_item_of_relation[1], items_of_item_of_relation[0], true);

            signed_items[0] = actual_item;
            signed_items[1] = symmetry_item;
            this.DrawRelation.Invalidate();
        }

        private void DrawRelationResize(object sender, EventArgs e)
        {
            this.DrawRelation.Invalidate();
        }
    }
}
