﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LibBinaryRelation.Controls
{
    public partial class TransitiveRelation : UserControl
    {
        public TransitiveRelation()
        {
            InitializeComponent();

            List<Brush> l = new List<Brush>() 
            {
                Brushes.Black,Brushes.Red,Brushes.Green,Brushes.Blue,Brushes.Brown,Brushes.Orange
            };

            set = new Set("X");
            relation = new Relation("R");

            l.ForEach(brush => set.AddItemToSet(brush));

            relation.AddItemToRelation(0, 1, Brushes.Black);
            relation.AddItemToRelation(2, 4, Brushes.Black);          
            relation.AddItemToRelation(0, 2, Brushes.Black);
            relation.AddItemToRelation(1, 3, Brushes.Black);
            relation.AddItemToRelation(4, 1, Brushes.Black);
            relation.AddItemToRelation(2, 3, Brushes.Black);
            relation.AddItemToRelation(2, 1, Brushes.Black);
            relation.AddItemToRelation(5, 3, Brushes.Black);          
            relation.AddItemToRelation(0, 3, Brushes.Black);
            relation.AddItemToRelation(4, 3, Brushes.Black);
            relation.AddItemToRelation(0, 4, Brushes.Black);

            graph1.SetGrid(6, 5);
            graph1.AddNode(Brushes.Black, 5, 2);
            graph1.AddNode(Brushes.Red, 4, 4);
            graph1.AddNode(Brushes.Green, 4, 0);
            graph1.AddNode(Brushes.Blue, 1, 4);
            graph1.AddNode(Brushes.Brown, 1, 0);
            graph1.AddNode(Brushes.Orange, 0, 2);
            this.graph1.AddArrow(0, 1, Brushes.LightGray);
            this.graph1.AddArrow(1, 3, Brushes.LightGray);
            this.graph1.AddArrow(0, 3, Brushes.LightGray);
            this.graph1.AddArrow(2, 4, Brushes.LightGray);
            this.graph1.AddArrow(4, 1, Brushes.LightGray);
            this.graph1.AddArrow(4, 3, Brushes.LightGray);
            this.graph1.AddArrow(2, 1, Brushes.LightGray);
            this.graph1.AddArrow(0, 2, Brushes.LightGray);
            this.graph1.AddArrow(0, 4, Brushes.LightGray);
            this.graph1.AddArrow(2, 3, Brushes.LightGray);
            this.graph1.AddArrow(5, 3, Brushes.LightGray);

            items = new int[30] { 0, 3, 8, 1, 4, 6, 1, 9, 5, 2, 1, 10, 2, 5, 8, 2, 6, 0, 4, 3, 9, 6, 3, 5, 10, 4, 0, 10, 9, 8 };
            signed_items = new List<int>() { 0, 3, 8 };
            actual_item = 0;
            colors = new List<Brush>() { Brushes.Black, Brushes.Black, Brushes.Red };
            SetItems();
        }

        Relation relation;
        Set set;
        int[] items;
        int actual_item;
        List<int> signed_items;
        List<Brush> colors;

        private void DescriptionSetPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            float v_space = p.Width * 0.05f;
            Drawing1.DrawSet(p, e.Graphics, new RectangleF(v_space, p.Height * 0.1f, p.Width - v_space, p.Height), set, this.Font);
        }

        private void DrawRelationPaint(object sender, PaintEventArgs e)
        {
            Panel p = (Panel)sender;
            Graphics g = e.Graphics;
            int v_space = (int)(p.Width * 0.05f);
            Rectangle rect = new Rectangle(v_space, (int)(p.Height * 0.1f), p.Width - 2 * v_space, p.Height);
            Drawing1.SignItemOfRelation(p, g, rect, relation, set, set, this.Font, signed_items, Pens.Black, Pens.Black, Pens.Red);
            Drawing1.DrawRelation(p, g, rect, set, set, relation, this.Font);
        }

        public Font GetMaxFont()
        {
            GroupBox l = this.DescriptionRelation;
            Graphics g = l.CreateGraphics();
            return Drawing1.GetFont(g, l.Width, l.Height, "\t" + l.Text + "\t", l.Font);
        }

        private void MoveRightClick(object sender, EventArgs e)
        {
            MoveLeft.Enabled = true;
            actual_item += 3;

            if (actual_item == items.Count() - 3) this.MoveRight.Enabled = false;
            SetItems();
        }

        private void MoveLeftClick(object sender, EventArgs e)
        {
            MoveRight.Enabled = true;
            actual_item -= 3;

            if (actual_item == 0) this.MoveLeft.Enabled = false;
            SetItems();           
        }

        void SetItems()
        {
            for (int i = 0; i < 3; i++)
            {
                List<int> items_of_item_of_relation = relation.GetItemOfRelation(signed_items[i]);
                this.graph1.SetWidthAndColorArrow(items_of_item_of_relation[0], items_of_item_of_relation[1], 1, Brushes.LightGray);
            }

            signed_items[0] = items[actual_item];
            signed_items[1] = items[actual_item + 1];
            signed_items[2] = items[actual_item + 2];

            for (int i = 0; i < 3; i++)
            {
                List<int> items_of_item_of_relation = relation.GetItemOfRelation(signed_items[i]);
                this.graph1.SetWidthAndColorArrow(items_of_item_of_relation[0], items_of_item_of_relation[1], 2, colors[i]);
            }
            this.DrawRelation.Invalidate();
        }

        private void DrawRelationResize(object sender, EventArgs e)
        {
            this.DescriptionSet.Invalidate();
            this.DrawRelation.Invalidate();
        }
    }
}
