﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using _3DTools;

namespace LibBinaryRelation
{
    public class Drawing3D
    {
        public static List<Point3D> GetIntersectionBallAndLine(
            Point3D p_ball, 
            double r_ball,
            Point3D a, 
            Point3D b)
        {
            Vector3D u = new Vector3D(b.X - a.X, b.Y - a.Y, b.Z - a.Z);

            double a_r = (u.X * u.X + u.Y * u.Y + u.Z * u.Z);
            double b_r = 2 * (a.X - p_ball.X) * u.X + 2 * (a.Y - p_ball.Y) * u.Y + 2 * (a.Z - p_ball.Z) * u.Z;
            double c_r = (a.X * a.X - 2 * a.X * p_ball.X + p_ball.X * p_ball.X) + (a.Y * a.Y - 2 * a.Y * p_ball.Y + p_ball.Y * p_ball.Y) + (a.Z * a.Z - 2 * a.Z * p_ball.Z + p_ball.Z * p_ball.Z) - r_ball * r_ball;

            double t1 = (-b_r + Math.Sqrt(b_r * b_r - 4 * a_r * c_r)) / (2 * a_r);
            double t2 = (-b_r - Math.Sqrt(b_r * b_r - 4 * a_r * c_r)) / (2 * a_r);

            Point3D inter1 = new Point3D(a.X + t1 * u.X, a.Y + t1 * u.Y, a.Z + t1 * u.Z);
            Point3D inter2 = new Point3D(a.X + t2 * u.X, a.Y + t2 * u.Y, a.Z + t2 * u.Z);

            return new List<Point3D>() { inter1, inter2 };
        }
        
        public static List<Ball> CreateBalls(List<Point3D> positions, List<Brush> brushes)
        {
            if (positions.Count != brushes.Count) return null;
            else
            {
                List<Ball> balls = new List<Ball>();

                for (int i = 0; i < brushes.Count; i++)
                {
                    balls.Add(new Ball());
                    balls[i].SetColor = brushes[i];
                    balls[i].Offset = positions[i];
                }

                return balls;
            }
        }

        public static Point3D Offset(Point3D point, double x, double y, double z)
        {
            return new Point3D(point.X + x, point.Y + y, point.Z + z);
        }

        public static List<Point3D> Offset(List<Point3D> points, double x, double y, double z)
        {
            List<Point3D> points1 = new List<Point3D>();
            points.ForEach(point => points1.Add(new Point3D(point.X + x, point.Y + y, point.Z + z)));
            return points1;
        }

        public static List<ScreenSpaceLines3D> Cube(List<Point3D> points, double thickness, Color color)
        {
            List<ScreenSpaceLines3D> cube1 = new List<ScreenSpaceLines3D>();
            cube1.Add(new ScreenSpaceLines3D());
            cube1[0].Color = color;
            cube1[0].Thickness = thickness;
            cube1[0].Points.Add(points[0]);
            cube1[0].Points.Add(points[1]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[1].Color = color;
            cube1[1].Thickness = thickness;
            cube1[1].Points.Add(points[0]);
            cube1[1].Points.Add(points[2]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[2].Color = color;
            cube1[2].Thickness = thickness;
            cube1[2].Points.Add(points[2]);
            cube1[2].Points.Add(points[3]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[3].Color = color;
            cube1[3].Thickness = thickness;
            cube1[3].Points.Add(points[1]);
            cube1[3].Points.Add(points[3]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[4].Color = color;
            cube1[4].Thickness = thickness;
            cube1[4].Points.Add(points[2]);
            cube1[4].Points.Add(points[4]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[5].Color = color;
            cube1[5].Thickness = thickness;
            cube1[5].Points.Add(points[0]);
            cube1[5].Points.Add(points[6]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[6].Color = color;
            cube1[6].Thickness = thickness;
            cube1[6].Points.Add(points[3]);
            cube1[6].Points.Add(points[5]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[7].Color =color;
            cube1[7].Thickness = thickness;
            cube1[7].Points.Add(points[1]);
            cube1[7].Points.Add(points[7]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[8].Color = color;
            cube1[8].Thickness = thickness;
            cube1[8].Points.Add(points[6]);
            cube1[8].Points.Add(points[7]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[9].Color = color;
            cube1[9].Thickness = thickness;
            cube1[9].Points.Add(points[6]);
            cube1[9].Points.Add(points[4]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[10].Color = color;
            cube1[10].Thickness = thickness;
            cube1[10].Points.Add(points[4]);
            cube1[10].Points.Add(points[5]);

            cube1.Add(new ScreenSpaceLines3D());
            cube1[11].Color = color;
            cube1[11].Thickness = thickness;
            cube1[11].Points.Add(points[7]);
            cube1[11].Points.Add(points[5]);

            return cube1;
        }

        static Point3D ReturnPointOnLine(Point3D a, Point3D b, double distance)
        {
            Vector3D u = new Vector3D(a.X - b.X, a.Y - b.Y, a.Z - b.Z);
            double distance_b_a = Math.Sqrt(Math.Pow(u.X, 2) + Math.Pow(u.Y, 2) + Math.Pow(u.Z, 2));
            double t = distance / distance_b_a;
            return new Point3D(b.X + t * u.X, b.Y + t * u.Y, b.Z + t * u.Z);
        }

        static List<Point3D> GetPointsForArrow(Point3D a, Point3D b, double length_arrow)
        {
            List<Point3D> points_for_arrow = new List<Point3D>();
            Point3D first_point = b;
            Point3D p = ReturnPointOnLine(a, b, length_arrow);
            //vektor primky l, urcene body a,b
            Vector3D u_l = new Vector3D(a.X - b.X, a.Y - b.Y, a.Z - b.Z);
            //primka p urcena bodem p a kolma k primce urcene body a,b
            Vector3D u_p;
            if (u_l.X == 0 && u_l.Z == 0) u_p = new Vector3D(-u_l.Y, 0, 0);
            else u_p = new Vector3D(-u_l.Z, 0, u_l.X);

            Vector3D u_p1 = -u_p;//new Vector3D(u_l.Z, 0, -u_l.X);
            Point3D left_point_p = new Point3D(p.X + 1 * u_p.X,p.Y + 1 * u_p.Y, p.Z + 1 * u_p.Z);
            Point3D right_point_p = new Point3D(p.X + 1 * u_p1.X,p.Y + 1 * u_p1.Y, p.Z + 1 * u_p1.Z);
            Point3D second_point = ReturnPointOnLine(left_point_p, p, length_arrow / 2);
            Point3D third_point = ReturnPointOnLine(right_point_p, p, length_arrow / 2);

            Vector3D u_s = Vector3D.CrossProduct(u_p, u_l);
            Point3D up_point = new Point3D(p.X + 1 * u_s.X, p.Y + 1 * u_s.Y, p.Z + 1 * u_s.Z);
            Point3D do_point = new Point3D(p.X + 1 * -u_s.X, p.Y + 1 * -u_s.Y, p.Z + 1 * -u_s.Z);
            Point3D fourth_point = ReturnPointOnLine(up_point, p, length_arrow / 2);
            Point3D fifth_point = ReturnPointOnLine(do_point, p, length_arrow / 2);
            points_for_arrow.Add(first_point);
            points_for_arrow.Add(second_point);
            points_for_arrow.Add(fourth_point);
            points_for_arrow.Add(third_point);
            points_for_arrow.Add(fifth_point);
            return points_for_arrow;
        }

        public static MeshGeometry3D CreateRectangleWithTextureCoor(Point3D p0, Point3D p1, Point3D p2, Point3D p3)
        {
            MeshGeometry3D mesh1 = new MeshGeometry3D();
            mesh1.Positions.Add(p0);
            mesh1.Positions.Add(p1);
            mesh1.Positions.Add(p2);
            mesh1.Positions.Add(p3);
            mesh1.TriangleIndices.Add(0);
            mesh1.TriangleIndices.Add(1);
            mesh1.TriangleIndices.Add(2);
            mesh1.TriangleIndices.Add(0);
            mesh1.TriangleIndices.Add(2);
            mesh1.TriangleIndices.Add(3);
            mesh1.TextureCoordinates.Add(new Point(0, 1));
            mesh1.TextureCoordinates.Add(new Point(1, 1));
            mesh1.TextureCoordinates.Add(new Point(1, 0));
            mesh1.TextureCoordinates.Add(new Point(0, 0));

            return mesh1;
        }

        public static GeometryModel3D CreateTriangleModel(Point3D p0, Point3D p1, Point3D p2, Brush color_triangle)
        {
            MeshGeometry3D mesh = new MeshGeometry3D();
            mesh.Positions.Add(p0);
            mesh.Positions.Add(p1);
            mesh.Positions.Add(p2);
            mesh.TriangleIndices.Add(0);
            mesh.TriangleIndices.Add(1);
            mesh.TriangleIndices.Add(2);
            Vector3D normal = CalculateNormal(p0, p1, p2);
            mesh.Normals.Add(normal);
            mesh.Normals.Add(normal);
            mesh.Normals.Add(normal);
            Material material = new DiffuseMaterial(color_triangle);
            GeometryModel3D model = new GeometryModel3D(
                mesh, material);
            return model;
        }

        public static Vector3D CalculateNormal(Point3D p0, Point3D p1, Point3D p2)
        {
            Vector3D v0 = new Vector3D(
                p1.X - p0.X, p1.Y - p0.Y, p1.Z - p0.Z);
            Vector3D v1 = new Vector3D(
                p2.X - p1.X, p2.Y - p1.Y, p2.Z - p1.Z);
            return Vector3D.CrossProduct(v0, v1);
        }

        public static Model3DGroup CreateArrowForLine(Point3D a, Point3D b, double length_arrow)
        {
            List<Point3D> points_for_arrow = GetPointsForArrow(a, b, length_arrow);
            Model3DGroup group = new Model3DGroup();
            for (int i = 1; i < points_for_arrow.Count - 1; i++)
            {
               group.Children.Add(CreateTriangleModel(points_for_arrow[0], points_for_arrow[i], points_for_arrow[i+1], Brushes.Red));
               group.Children.Add(CreateTriangleModel(points_for_arrow[0], points_for_arrow[i+1], points_for_arrow[i], Brushes.Red));
            }

            group.Children.Add(CreateTriangleModel(points_for_arrow[0], points_for_arrow[1], points_for_arrow[points_for_arrow.Count - 1], Brushes.Red));
            group.Children.Add(CreateTriangleModel(points_for_arrow[0], points_for_arrow[points_for_arrow.Count - 1], points_for_arrow[1], Brushes.Red));

            return group;
        }


    }

    //bezier curve
    public class ScreenSpaceBezierCurve : ModelVisual3D
    {
        public ScreenSpaceBezierCurve(Point3D point0, Point3D point1, Point3D point2, Point3D point3, Color color)
        {
            List<Point3D> points = GetPointsBezierCurve(point0, point1, point2, point3);
            for (int i = 0; i < points.Count - 1; i++)
            {
                ScreenSpaceLines3D s = new ScreenSpaceLines3D();
                s.Thickness = 1;
                s.Color = color;
                s.Points.Add(points[i]);
                s.Points.Add(points[i + 1]);
                this.Children.Add(s);
            }
        }

        List<Point3D> GetPointsBezierCurve(Point3D point0, Point3D point1, Point3D point2, Point3D point3)
        {
            List<Point3D> points = new List<Point3D>();

            for (double t = 0; t <= 1; t += 0.01)
            {
                points.Add(new Point3D( 
                    (1 - Math.Pow(t, 3)) * point0.X + 3 * Math.Pow((1 - t), 2) * t * point1.X + 3 * (1 - t) * Math.Pow(t, 2) * point2.X + Math.Pow(t, 3) * point3.X,
                    (1 - Math.Pow(t, 3)) * point0.Y + 3 * Math.Pow((1 - t), 2) * t * point1.Y + 3 * (1 - t) * Math.Pow(t, 2) * point2.Y + Math.Pow(t, 3) * point3.Y,
                    (1 - Math.Pow(t, 3)) * point0.Z + 3 * Math.Pow((1 - t), 2) * t * point1.Z + 3 * (1 - t) * Math.Pow(t, 2) * point2.Z + Math.Pow(t, 3) * point3.Z));
                    
            }

            return points;
        }
    }

    //ball
    public class Ball : ModelVisual3D
    {
        public Ball()
        {
            this.Content = new GeometryModel3D();
            (this.Content as GeometryModel3D).Geometry = Tessellate();
            number_of_item = -1;
            number_of_set = -1;
        }

        int number_of_item;
        int number_of_set;

        public int NumberOfItem
        {
            set { this.number_of_item = value; }
            get { return this.number_of_item; }
        }

        public int NumberOfSet
        {
            set { this.number_of_set = value; }
            get { return this.number_of_set; }
        }

        internal Point3D GetPosition(double t, double y)
        {
            double r = Math.Sqrt(1 - y * y);
            double x = r * Math.Cos(t);
            double z = r * Math.Sin(t);

            return new Point3D(x, y, z);
        }

        private Vector3D GetNormal(double t, double y)
        {
            return (Vector3D)GetPosition(t, y);
        }

        private Point GetTextureCoordinate(double t, double y)
        {
            Matrix TYtoUV = new Matrix();
            TYtoUV.Scale(1 / (2 * Math.PI), -0.5);

            Point p = new Point(t, y);
            p = p * TYtoUV;

            return p;
        }

        /*public string ImageSource
        {
            set
            {

                DiffuseMaterial dm = new DiffuseMaterial();
                ImageSource imSrc = new
                BitmapImage(new Uri(value, UriKind.RelativeOrAbsolute));
                dm.Brush = new ImageBrush(imSrc);

                (this.Content as GeometryModel3D).Material = dm;
            }
        }*/

        public Brush SetColor
        {
            set
            {
                (this.Content as GeometryModel3D).Material = new DiffuseMaterial(value);
            }
        }

        public Point3D Offset
        {
            set
            {
                this.Transform = new
                    TranslateTransform3D(value.X, value.Y, value.Z);
            }
        }

        internal Geometry3D Tessellate()
        {
            int tDiv = 20;
            int yDiv = 20;
            double maxTheta = 3.14 * (360.0 / 180.0);//MathHelper.DegToRad(360.0);
            double minY = -1.0;
            double maxY = 1.0;

            double dt = maxTheta / tDiv;
            double dy = (maxY - minY) / yDiv;

            MeshGeometry3D mesh = new MeshGeometry3D();

            for (int yi = 0; yi <= yDiv; yi++)
            {
                double y = minY + yi * dy;

                for (int ti = 0; ti <= tDiv; ti++)
                {
                    double t = ti * dt;

                    mesh.Positions.Add(GetPosition(t, y));
                    mesh.Normals.Add(GetNormal(t, y));
                    mesh.TextureCoordinates.Add(GetTextureCoordinate(t, y));
                }
            }

            for (int yi = 0; yi < yDiv; yi++)
            {
                for (int ti = 0; ti < tDiv; ti++)
                {
                    int x0 = ti;
                    int x1 = (ti + 1);
                    int y0 = yi * (tDiv + 1);
                    int y1 = (yi + 1) * (tDiv + 1);

                    mesh.TriangleIndices.Add(x0 + y0);
                    mesh.TriangleIndices.Add(x0 + y1);
                    mesh.TriangleIndices.Add(x1 + y0);

                    mesh.TriangleIndices.Add(x1 + y0);
                    mesh.TriangleIndices.Add(x0 + y1);
                    mesh.TriangleIndices.Add(x1 + y1);
                }
            }

            mesh.Freeze();
            return mesh;
        }
    }
}
